#version 150 core
//es2:#extension GL_OES_standard_derivatives:enable

//  Copyright © 2013-2017, 2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

in vec3 worldPosition;
in vec3 worldNormal;
in vec4 color;
in vec2 texcoord;
in vec3 barycentric;

out vec4 fragColor;

uniform vec3 eyePosition;
uniform sampler2D tex;

vec3 bevel_color = vec3(15./255.);

#pragma include phong.inc.frag

void main()
{
    vec3 col;
    vec3 worldView = normalize(eyePosition - worldPosition);
    vec4 col_tex = texture2D(tex, texcoord);
    float bary_min = min(min(barycentric.x, barycentric.y), barycentric.z);
    //es2:#ifdef GL_OES_standard_derivatives
    float bary_width = fwidth(bary_min);
    //es2:#endif
    if (barycentric == vec3(0.)) {
        // unlabeled part
        col = bevel_color;
    } else if (bary_min <= 0.02) {
        // frame around the label
        col = bevel_color;
    } else {
        // the label
        col = mix(color.rgb, col_tex.rgb, col_tex.a);
        //es2:#ifdef GL_OES_standard_derivatives
        bary_min = (bary_min - 0.02) / 1.1;
        if (bary_min < bary_width) {
            // smooth at the frame
            col = mix(bevel_color, col, bary_min / bary_width);
        }
        //es2:#endif
    }
    
    fragColor = phongFunction(vec4(0.8), vec4(col, 1.), vec4(0.4), 2.0, worldPosition, worldView, worldNormal);
}

