// This file was taken from the Qt3D module of the Qt Toolkit.
// From the license alternatives the BSD License Usage was choosen as
// the license of this file.
//---- Copyright of the changes:
//  Copyright © 2019  B. Clausius <barcc@gmx.de>
//---- Original Copyright and License Statement from the Qt Toolkit:
/****************************************************************************
**
** Copyright (C) 2017 Klaralvdalens Datakonsult AB (KDAB).
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#pragma include light.inc.frag

void adsModel(const in vec3 worldPos,
              const in vec3 worldNormal,
              const in vec3 worldView,
              const in float shininess,
              out vec3 diffuseColor,
              out vec3 specularColor)
{
    diffuseColor = vec3(0.0);
    specularColor = vec3(0.0);

    // We perform all work in world space
    vec3 n = normalize(worldNormal);
    vec3 s = vec3(0.0); //gl3.

    //es2:vec3 s;
    //es2:Light light;
    for (int i = 0; i < lightCount; ++i) {
        //es2:if (i == 0)
        //es2:    light = lights[0];
        //es2:else if (i == 1)
        //es2:    light = lights[1];
        //es2:else if (i == 2)
        //es2:    light = lights[2];
        //es2:else if (i == 3)
        //es2:    light = lights[3];
        //es2:else if (i == 4)
        //es2:    light = lights[4];
        //es2:else if (i == 5)
        //es2:    light = lights[5];
        //es2:else if (i == 6)
        //es2:    light = lights[6];
        //es2:else if (i == 7)
        //es2:    light = lights[7];

        float att = 1.0;
        //es2:if ( light.type != TYPE_DIRECTIONAL ) {
        //es2:    s = light.position - worldPos;
        //es2:    if (length( light.attenuation ) != 0.0) {
        //es2:        float dist = length(s);
        //es2:        att = 1.0 / (light.attenuation.x + light.attenuation.y * dist + light.attenuation.z * dist * dist);
        //es2:    }
        //es2:    s = normalize( s );
        //es2:    if ( light.type == TYPE_SPOT ) {
        //es2:        if ( degrees(acos(dot(-s, normalize(light.direction))) ) > light.cutOffAngle)
        //es2:            att = 0.0;
        float sDotN = 0.0; //gl3.

        if (lights[i].type != TYPE_DIRECTIONAL) { //gl3.
            // Point and Spot lights //gl3.

            // Light position is already in world space //gl3.
            vec3 sUnnormalized = lights[i].position - worldPos; //gl3.
            s = normalize(sUnnormalized); // Light direction //gl3.

            // Calculate the attenuation factor //gl3.
            sDotN = dot(s, n); //gl3.
            if (sDotN > 0.0) { //gl3.
                if (lights[i].constantAttenuation != 0.0 //gl3.
                 || lights[i].linearAttenuation != 0.0 //gl3.
                 || lights[i].quadraticAttenuation != 0.0) { //gl3.
                    float dist = length(sUnnormalized); //gl3.
                    att = 1.0 / (lights[i].constantAttenuation + //gl3.
                                 lights[i].linearAttenuation * dist + //gl3.
                                 lights[i].quadraticAttenuation * dist * dist); //gl3.
                } //gl3.

                // The light direction is in world space already //gl3.
                if (lights[i].type == TYPE_SPOT) { //gl3.
                    // Check if fragment is inside or outside of the spot light cone //gl3.
                    if (degrees(acos(dot(-s, lights[i].direction))) > lights[i].cutOffAngle) //gl3.
                        sDotN = 0.0; //gl3.
                } //gl3.
            }
        } else {
            // Directional lights
            // The light direction is in world space already
            //es2:s = normalize( -light.direction );
            s = normalize(-lights[i].direction); //gl3.
            sDotN = dot(s, n); //gl3.
        }

        // Calculate the diffuse factor
        //es2:float diffuse = max( dot( s, n ), 0.0 );
        float diffuse = max(sDotN, 0.0); //gl3.

        // Calculate the specular factor
        float specular = 0.0;
        //es2:if (diffuse > 0.0 && shininess > 0.0 && att > 0.0) {
        if (diffuse > 0.0 && shininess > 0.0) { //gl3.
            float normFactor = (shininess + 2.0) / 2.0;
            vec3 r = reflect(-s, n);   // Reflection direction in world space
            specular = normFactor * pow(max(dot(r, worldView), 0.0), shininess);
        }

        // Accumulate the diffuse and specular contributions
        //es2:diffuseColor += att * light.intensity * diffuse * light.color;
        //es2:specularColor += att * light.intensity * specular * light.color;
        diffuseColor += att * lights[i].intensity * diffuse * lights[i].color; //gl3.
        specularColor += att * lights[i].intensity * specular * lights[i].color; //gl3.
    }
}

vec4 phongFunction(const in vec4 ambient,
                   const in vec4 diffuse,
                   const in vec4 specular,
                   const in float shininess,
                   const in vec3 worldPosition,
                   const in vec3 worldView,
                   const in vec3 worldNormal)
{
    // Calculate the lighting model, keeping the specular component separate
    vec3 diffuseColor, specularColor;
    adsModel(worldPosition, worldNormal, worldView, shininess, diffuseColor, specularColor);

    // Combine spec with ambient+diffuse for final fragment color
    vec3 color = (ambient.rgb + diffuseColor) * diffuse.rgb
               + specularColor * specular.rgb;

    return vec4(color, diffuse.a);
}
