//  Copyright © 2015-2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.4
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.3

FocusScope {
    id: root
    
    Item {
        anchors.fill: parent
        
        Rectangle {
            id: header
            width: parent.width
            height: headerrow.implicitHeight
            color: palette.window
            
            RowLayout {
                id: headerrow
                anchors.fill: parent
                
                ToolButton {
                    id: backbutton
                    contentItem: Image {
                        fillMode: Image.Pad
                        horizontalAlignment: Image.AlignHCenter
                        verticalAlignment: Image.AlignVCenter
                        source: Qt.resolvedUrl("images/go-previous.png")
                    }
                    onClicked: { ctx.aboutdialog_visible = false; root.StackView.view.pop() }
                }
                Image {
                    id: appicon
                    fillMode: Image.Pad
                    source: Qt.resolvedUrl(ctx.config("APPICON_FILE"))
                }
                Column {
                    id: col_apptext
                    Layout.fillWidth: true
                    spacing: 10
                    
                    Row {
                        anchors.horizontalCenter: parent.horizontalCenter
                        spacing: 20
                        
                        Label {
                            id: appname
                            font.pointSize: 16
                            font.weight: Font.DemiBold
                            font.bold: true
                            text: ctx.config_tr("APPNAME")
                            textFormat: Text.PlainText
                        }
                        Label {
                            id: version
                            anchors.baseline: appname.baseline
                            text: ctx.config("VERSION")
                            textFormat: Text.PlainText
                    }   }
                    Label {
                        id: description
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: ctx.config_tr("SHORT_DESCRIPTION")
                        textFormat: Text.PlainText
                        wrapMode: Text.WordWrap
        }   }   }   }
        Rectangle {
            id: body
            anchors.top: header.bottom
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            implicitWidth: tabview.implicitWidth
            implicitHeight: tabview.implicitHeight
            color: palette.window
            
            TabBar {
                id: tabbar
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: parent.right
                TabButton { text: ctx.tr("About") }
                TabButton { text: ctx.tr("Contribute") }
                TabButton { text: ctx.tr("License") }
            }
            StackLayout {
                id: tabview
                anchors.top: tabbar.bottom
                anchors.bottom: parent.bottom
                anchors.left: parent.left
                anchors.right: parent.right
                currentIndex: tabbar.currentIndex
                
                Item {
                    id: tab_about
                    property bool anim_running: false
                    onVisibleChanged: anim_running = true
                    Component.onCompleted: anim_running = true
                    
                    Item {
                        anchors.fill: parent
                        Column {
                            id: labels_about
                            anchors.margins: 6
                            anchors.top: parent.top
                            anchors.left: parent.left
                            anchors.right: parent.right
                            spacing: 6
                            
                            Label {
                                textFormat: Text.PlainText
                                text: ctx.config("COPYRIGHT")
                            }
                            Label {
                                text: ctx.dialog_text("about_website")
                                textFormat: Text.StyledText
                                MouseArea {
                                    anchors.fill: parent
                                    acceptedButtons: Qt.NoButton
                                    cursorShape: Qt.PointingHandCursor
                                }
                                onLinkActivated: Qt.openUrlExternally(link)
                            }
                            Label {
                                text: ctx.tr("Translators:")
                                textFormat: Text.PlainText
                        }   }
                        Frame {
                            anchors.top: labels_about.bottom
                            anchors.bottom: parent.bottom
                            width: parent.width
                            MouseArea {
                                z: 10
                                anchors.fill: parent
                                enabled: tab_about.anim_running
                                cursorShape: enabled ? Qt.ArrowCursor: translators_text.hoveredLink=="" ?
                                                            Qt.IBeamCursor : Qt.PointingHandCursor
                                onPressed: { mouse.accepted = true; tab_about.anim_running = false }
                                onWheel: { wheel.accepted = false; tab_about.anim_running = false }
                            }
                            Flickable {
                                id: translators
                                anchors.fill: parent
                                TextArea.flickable: TextArea {
                                    id: translators_text
                                    text: ctx.dialog_text("about_translators")
                                    textFormat: Text.RichText
                                    readOnly: true
                                    onLinkActivated: Qt.openUrlExternally(link)
                                }
                                ScrollIndicator.vertical: ScrollIndicator {}
                                property real anim_len: contentHeight - height + 100.0
                                property real anim_pos: 0.0
                                contentY: anim_pos*anim_len - 50.0
                            }
                            SequentialAnimation {
                                running: tab_about.visible && tab_about.anim_running && (translators.anim_len>100.0)
                                loops: Animation.Infinite
                                
                                NumberAnimation {
                                    target: translators
                                    property: "anim_pos"
                                    from: 0.0
                                    to: 1.0
                                    // values are updated only if not running,
                                    // so duration is wrong if translators resized
                                    duration: Math.max(1000, translators.anim_len * 16.0)
                                    easing.type: Easing.InOutSine
                                }
                                NumberAnimation {
                                    target: translators
                                    property: "anim_pos"
                                    from: 1.0
                                    to: 0.0
                                    duration: Math.max(1000, translators.anim_len * 16.0)
                                    easing.type: Easing.InOutSine
                            }   }
                }   }   }
                Item {
                    TextArea {
                        anchors.fill: parent
                        text: ctx.dialog_text("about_contribute")
                        readOnly: true
                        textFormat: Text.RichText
                        wrapMode: Text.WordWrap
                        
                        onLinkActivated: Qt.openUrlExternally(link)
                }   }
                Flickable {
                    id: tab_license
                    
                    property bool short_text: true
                    
                    ScrollBar.vertical: ScrollBar {}
                    
                    TextArea.flickable: TextArea {
                        anchors.fill: parent
                        function get_license_text(short_text) {
                            if (short_text) {
                                wrapMode = TextEdit.WordWrap
                                return ctx.dialog_text("about_license_short")
                            }
                            var longtext = ctx.dialog_text("about_license_full")
                            if (longtext) {
                                wrapMode = TextEdit.NoWrap
                                return longtext
                            }
                            wrapMode = TextEdit.WordWrap
                            return ctx.dialog_text("about_license_notfound")
                            
                        }
                        text: get_license_text(tab_license.short_text)
                        readOnly: true
                        textFormat: Text.RichText
                        wrapMode: TextEdit.WordWrap
                        
                        onLinkActivated: {
                            if (link.indexOf("text:")===0)
                                tab_license.short_text = false
                            else
                                Qt.openUrlExternally(link)
                        }
                    }
                    onVisibleChanged: { if (visible)  short_text = true }
                }
            }
        }
    }
    Keys.onReleased: {
        if (event.key == Qt.Key_Return || event.key == Qt.Key_Enter
         || event.key == Qt.Key_Escape || event.key == Qt.Key_Back) {
            event.accepted = true
            ctx.aboutdialog_visible = false
            StackView.view.pop()
    }   }
}

