//  Copyright © 2017-2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.7
import QtQuick.Controls 2.0
import QtQuick.Window 2.2
import "." as Pybik

Item {
    focus: true
    activeFocusOnTab: true
    
    property real extraheight: 0
    property real drawingheight: height + extraheight
    
    Keys.onPressed: event.accepted = ctx.key_pressed(event.key, event.modifiers, true)
    Connections {
        target: ctx
        onAnimation_angleChanged: { if (animation.running) ctx.animation_step(ctx.animation_angle) }
        onAnimation_runningChanged: { animation.running = ctx.animation_running }
    }
    
    NumberAnimation {
        id: animation
        target: ctx
        property: "animation_angle"
        running: false
        to: ctx.animation_maxangle
        duration:  ctx.animation_maxangle / ctx.animation_speed * 100
        onStopped: { ctx.animation_running = false; ctx.animation_angle = 0; ctx.animation_stopped() }
    }
    Text {
        visible: ctx.debugmsg != ""
        x: 4
        y: 4
        z: 1
        color: "black"
        text: ctx.debugmsg
        textFormat: Text.PlainText
    }
    DropArea {
        anchors.fill: parent
        onEntered: {
            if (drag.hasColor)  drag.acceptProposedAction()
            else if (drag.hasUrls && drag.urls[0].split(":")[0]=="file")  drag.acceptProposedAction()
        }
        onDropped: {
            if (drop.hasColor) {
                ctx.drag_dropped_color(drop.colorData)
                scene.request_pick(drop.x, drop.y)
            } else if (drop.hasUrls) {
                ctx.drag_dropped_url(drop.urls[0])
                scene.request_pick(drop.x, drop.y)
            }
        }
    }
    PinchArea {
        z: -1
        anchors.fill: parent
        onPinchUpdated: {
            ctx.drawingarea_zoom((pinch.previousScale - pinch.scale)*10)
        }
    }
    Pybik.Scene {
        id: scene
        anchors.fill: parent
}   }

