//  Copyright © 2015-2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.7
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.1
import QtQuick.Dialogs 1.2
import "." as Pybik

FocusScope {
    id: root
    
    Component.onCompleted: drawingarea.forceActiveFocus()
    
    SystemPalette { id: palette }
    
    Shortcut {
        sequence: StandardKey.Quit
        context: Qt.ApplicationShortcut
        onActivated: ctx.action("quit")
    }
    Shortcut {
        sequence: ctx.get_settings("action.initial_state")
        enabled: root.visible
        onActivated: ctx.action("initial_state")
    }
    Shortcut {
        sequence: ctx.get_settings("action.reset_rotation")
        enabled: root.visible
        onActivated: ctx.action("reset_rotation")
    }
    Shortcut {
        sequence: ctx.get_settings("action.preferences")
        enabled: root.visible
        onActivated: ctx.action("preferences")
    }
    Shortcut {
        sequence: "F1"
        onActivated: ctx.helpdialog_visible = true
    }
    Shortcut {
        sequence: ctx.get_settings("action.edit_moves")
        enabled: editbar.visible
        onActivated: editbartext.forceActiveFocus()
    }
    Shortcut {
        sequence: ctx.get_settings("action.edit_cube")
        enabled: root.visible
        onActivated: ctx.action("edit_cube")
    }
    
    ToolBar {
        id: toolbar
        z: 1
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        
        Keys.onPressed: event.accepted = ctx.key_pressed(event.key, event.modifiers, false)
        
        RowLayout {
            id: toolbarlayout1
            anchors.fill: parent
            spacing: 4
            
            ToolButton {
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/open-menu.png")
                }
                onClicked: { sidepane.visible = !sidepane.visible }
            }
            Item {
                Layout.fillWidth: true
                Label {
                    id: statustext
                    anchors.centerIn: parent
                    text: ctx.status
                    textFormat: Text.PlainText
                    elide: Text.ElideRight
            }   }
            ToolButton { 
                id: action_challenge
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/new-random.png")
                }
                enabled: root.visible
                Shortcut {
                    sequence: "Ctrl+N"
                    onActivated: action_challenge.onClicked()
                }
                onClicked: ctx.action("challenge")
            }
            ToolButton { 
                id: action_new_solved
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/new-solved.png")
                }
                enabled: root.visible
                Shortcut {
                    sequence: "Ctrl+Shift+N"
                    onActivated: action_new_solved.onClicked()
                }
                onClicked: ctx.action("new_solved")
    }   }   }
    Rectangle {
        id: editbar
        visible: ctx.get_settings("window.editbar")
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: toolbar.bottom
        height: visible ? implicitHeight : 0
        implicitHeight: editbartext.implicitHeight
        
        Keys.onPressed: event.accepted = ctx.key_pressed(event.key, event.modifiers, false)
        
        RowLayout {
            anchors.fill: parent
            spacing: 0
            //TODO: use TextField for editable text, see revision 4614
            Text {
                id: editbartext
                Layout.fillWidth: true
                text: ""
                textFormat: Text.StyledText
                Binding on text { value: ctx.edittext1+'<i><font color="gray">'+ctx.edittext2+'</font></i>' }
    }   }   }
    // solved message, etc.
    Popup {
        id: messagebox
        x: (parent.width - width) / 2
        y: (parent.height - height) / 2
        width: Math.min(parent.width - 40, 320)
        height: messagebox_text.paintedHeight + 16
        modal: true
        
        Connections {
            target: ctx
            onMessagebox_visibleChanged: { if (ctx.messagebox_visible)  messagebox.open() }
        }
        onClosed: ctx.messagebox_visible = false
        
        Text {
            id: messagebox_text
            color: palette.buttonText
            anchors.margins: 12
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: parent.left
            anchors.right: parent.right
            text: ctx.messagetext
            textFormat: Text.PlainText
            wrapMode: Text.WordWrap
            horizontalAlignment: Text.AlignHCenter
    }   }
    Pybik.DrawingArea {
        id: drawingarea
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: editbar.bottom
        anchors.bottom: parent.bottom
        extraheight: bottombar.height
    }
    // bottom bar, challenges, status text
    ToolBar {
        id: bottombar
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        background: Rectangle {
            gradient: Gradient {
                GradientStop { position: 0.0; color: "transparent" }
                GradientStop { position: 1.0; color: "white" }
        }   }
        RowLayout {
            id: bottomlayout
            anchors.fill: parent
            Item { Layout.fillWidth: true }
            ToolButton {
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-seek-backward.png")
                }
                background: null
                enabled: ctx.toolbarstate[0]
                onClicked: ctx.action("rewind")
            } ToolButton {
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-skip-backward.png")
                }
                background: null
                enabled: ctx.toolbarstate[0]
                onClicked: ctx.action("previous")
            } ToolButton {
                visible: ctx.toolbarstate[3]
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-playback-stop.png")
                }
                background: null
                onClicked: ctx.action("stop")
            } ToolButton {
                visible: !ctx.toolbarstate[3]
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-playback-start.png")
                }
                background: null
                enabled: ctx.toolbarstate[1]
                onClicked: ctx.action("play")
            } ToolButton {
                id: tool_next
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-skip-forward.png")
                }
                background: null
                enabled: ctx.toolbarstate[1]
                onClicked: ctx.action("next")
            } ToolButton {
                id: tool_forward
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/media-seek-forward.png")
                }
                background: null
                enabled: ctx.toolbarstate[1]
                onClicked: ctx.action("forward")
            } ToolButton {
                id: tool_mark_set
                visible: ctx.toolbarstate[4]
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/list-add.png")
                }
                background: null
                enabled: ctx.toolbarstate[2]
                onClicked: ctx.action("mark_set")
            } ToolButton {
                id: tool_mark_remove
                visible: !ctx.toolbarstate[4]
                contentItem: Image {
                    fillMode: Image.Pad
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    source: Qt.resolvedUrl("images/list-remove.png")
                }
                background: null
                enabled: ctx.toolbarstate[2]
                onClicked: ctx.action("mark_remove")
            }
            Item { width: tool_mark_remove.height/2 }
    }   }
    Drawer {
        id: sidepane
        width: Math.min(parent.width/3*2, Math.max(parent.width/3, 280))
        height: parent.height
        edge: Qt.LeftEdge
        //FIXME: QTBUG-59141: Buttons / mouseArea doesn't works under Drawer's drag area
        dragMargin: 0
        Column {
            id: sidepanebuttons
            width: parent.width
            Button {
                id: action_selectmodel
                width: parent.width
                flat: true
                text: ctx.tr("Select Model …")
                icon.source: Qt.resolvedUrl("images/select-model.png")
                icon.color: "transparent"
                Shortcut {
                    sequence: ctx.get_settings("action.selectmodel")
                    enabled: root.visible
                    onActivated: action_selectmodel.onClicked()
                }
                onClicked: {sidepane.close(); ctx.model_start_selection(); push(Qt.resolvedUrl("ModelPage.qml"))}
            } ToolSeparator {
                width: parent.width
                orientation: Qt.Horizontal
            } Button {
                width: parent.width
                flat: true
                text: ctx.tr("Reset Rotation")
                onClicked: {sidepane.close(); ctx.action("reset_rotation")}
            } Button {
                width: parent.width
                flat: true
                text: ctx.tr("Set as Initial State")
                onClicked: {sidepane.close(); ctx.action("initial_state")}
            } CheckBox {
                width: parent.width
                checkable: true
                checked: editbar.visible
                text: ctx.tr("Show Move Sequence")
                onClicked: {sidepane.close(); editbar.visible = !editbar.visible; ctx.set_settings("window.editbar", editbar.visible) }
            } ToolSeparator {
                width: parent.width
                orientation: Qt.Horizontal
            } Button {
                width: parent.width
                flat: true
                text: ctx.tr("Preferences …")
                onClicked: {sidepane.close(); ctx.action("preferences")}
            } Button {
                width: parent.width
                flat: true
                text: ctx.tr("Info …")
                onClicked: {sidepane.close(); ctx.aboutdialog_visible = true}
            } Button {
                width: parent.width
                flat: true
                text: ctx.tr("Help …")
                onClicked: {sidepane.close(); ctx.helpdialog_visible = true}
            } ToolSeparator {
                width: parent.width
                orientation: Qt.Horizontal
            }
        }
        Pybik.PluginPane {
            width: parent.width
            anchors.top: sidepanebuttons.bottom
            anchors.bottom: parent.bottom
        }
    }
}


