//  Copyright © 2015-2017  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.3
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.1
import QtQuick.Window 2.2

FocusScope {
    id: root
    focus: true
    property real scalefactor: Math.max(Math.min(Math.min(Math.min(width, height)/3,
                                                          Math.max(width, height)/4) / 128,
                                                 1.5),
                                        1/1.6)
    //property real scalefactor: Math.max(Math.min(Math.sqrt(width*height/12.) / 128,
    //                                             1.5),
    //                                    1/1.6)
    
    Rectangle {
        anchors.fill: parent
        color: palette.window
        focus: true
        
        SystemPalette { id: palette }
        
        ColumnLayout {
            id: selectlayout
            anchors.fill: parent
            focus: true
            spacing: 0
            
            ToolBar {
                Layout.fillWidth: true
                z: 1
                RowLayout {
                    anchors.fill: parent
                    ToolButton {
                        contentItem: Image {
                            fillMode: Image.Pad
                            horizontalAlignment: Image.AlignHCenter
                            verticalAlignment: Image.AlignVCenter
                            source: Qt.resolvedUrl("images/go-previous.png")
                        }
                        onClicked: { ctx.model_back(); root.StackView.view.pop() }
                    }
                    Label {
                        id: title
                        Layout.fillWidth: true
                        horizontalAlignment: Text.AlignHCenter
                        text: ''
            }   }   }
            GridView {
                id: gridview
                Layout.fillWidth: true
                Layout.fillHeight: true
                focus: true
                keyNavigationWraps: true
                onModelChanged: { title.text = ctx.selectmodel_title; currentIndex = ctx.selectmodel_index }
                cellWidth: 128*scalefactor
                cellHeight: 128*scalefactor
                Component.onCompleted: {
                    forceActiveFocus()
                    // root.StackView.index == -1, use Timer to set model immediately afterwards
                    setmodeltimer.running = true
                }
                onWidthChanged: {
                    positionViewAtIndex(currentIndex, GridView.Center)
                    cellWidth = width/Math.floor(width/128./scalefactor)
                }
                
                Timer {
                    id: setmodeltimer
                    interval: 0
                    onTriggered: gridview.model = ctx.model_get_modeldata(root.StackView.index)
                }
                
                delegate: Image {
                    width: gridview.cellWidth
                    height: 128*scalefactor
                    fillMode: Image.PreserveAspectFit
                    source: modelData.key
                    Rectangle {
                        anchors.bottom: parent.bottom
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.margins: 4*scalefactor
                        height: modeltext.height
                        width: modeltext.contentWidth+6
                        color: "#66000000"
                        radius: 4*scalefactor
                        Text {
                            id: modeltext
                            anchors.bottom: parent.bottom
                            anchors.horizontalCenter: parent.horizontalCenter
                            width: (128-10)*scalefactor
                            text: modelData.text
                            color: "white"
                            wrapMode: Text.Wrap
                            textFormat: Text.PlainText
                            horizontalAlignment: Text.AlignHCenter
                    }   }
                    MouseArea {
                        hoverEnabled: true
                        anchors.fill: parent
                        anchors.margins: 1 //avoid spurious scroll if mouse moves to adjacent items
                        onEntered: {
                            if (gridview.contains(mapToItem(gridview, mouseX, mouseY)))
                                parent.GridView.view.currentIndex = index
                        }
                        onClicked: {
                            if (ctx.model_selected(root.StackView.index, index))
                                root.StackView.view.push(Qt.resolvedUrl("ModelPage.qml"))
                            else
                                root.StackView.view.pop(null)
                }   }   }
                highlight: Rectangle {
                    radius: 8*scalefactor
                    color: "#55003070"
            }   }
        }
    }
    //TODO: return
    Keys.onReleased: {
        if (event.key == Qt.Key_Escape || event.key == Qt.Key_Back) {
            event.accepted = true
            ctx.preferences_visible = false
            StackView.view.pop()
    }   }
}

