//  Copyright © 2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQml 2.0
import QtQuick.Scene3D 2.0
import Qt3D.Core 2.0
import Qt3D.Render 2.11
import Qt3D.Input 2.0
import Qt3D.Logic 2.0

Scene3D {
    focus: true
    aspects: ["render", "logic", "input"]
    cameraAspectRatioMode: Scene3D.AutomaticAspectRatio
    hoverEnabled: ctx.hover_enabled()
    
    function request_pick(x, y) {
        screenraycaster.trigger(Qt.point(x, y))
    }
    
    Entity {
        components: [
            RenderSettings {
                pickingSettings.pickMethod: PickingSettings.TrianglePicking
                activeFrameGraph: RenderSurfaceSelector {
                    CameraSelector {
                        camera: camera
                        Viewport {
                            ClearBuffers {
                                clearColor: ctx.scene_bgcolor
                                buffers: ClearBuffers.ColorDepthBuffer
                }   }   }   }
            },
            InputSettings {}
        ]
        
        FrameAction {
            onTriggered: {
                ctx.frame_triggered()
            }
        }
        Camera {
            id: camera
            projectionType: CameraLens.PerspectiveProjection
            fieldOfView: 44.1445/(ctx.zoom+0.269595) + 0.559741
            nearPlane : 0.1
            farPlane : 150.0
            position: Qt.vector3d( 0.0, 0.0, 4*ctx.bounding_sphere_radius)
            upVector: Qt.vector3d( 0.0, 1.0, 0.0 )
            viewCenter: Qt.vector3d( 0.0, 0.0, 0.0 )
        }
        Transform {
            id: puzzleTransform
            matrix: {
                var m = Qt.matrix4x4();
                m.rotate(ctx.rotationy, Qt.vector3d(1, 0, 0));
                m.rotate(ctx.rotationx, Qt.vector3d(0, 1, 0));
                return m;
        }   }
        MouseHandler {
            id: mousehandler
            sourceDevice: MouseDevice {}
            onPressed: {
                ctx.mouse_pressed(mouse.modifiers, mouse.button, mouse.x, mouse.y)
                screenraycaster.trigger(Qt.point(mouse.x, mouse.y))
            }
            onReleased: ctx.drawingarea_mouse_released(mouse.x, mouse.y)
            onPositionChanged: {
                ctx.drawingarea_mouse_moved(mouse.buttons, mouse.x, mouse.y)
                if (ctx.hover_enabled() && !mouse.buttons)
                    screenraycaster.trigger(Qt.point(mouse.x, mouse.y))
            }
            onWheel: ctx.drawingarea_zoom(wheel.angleDelta.y/120.0)
        }
        ScreenRayCaster {
            id: screenraycaster
            onHitsChanged: {
                var dist=-1, hit
                for (var i=0; i<hits.length; i++) {
                    if (dist<0 || dist>hits[i].distance) {
                        dist = hits[i].distance
                        hit = hits[i]
                }   }
                var idx, wi, pos = Qt.point(position.x/width*2-1, 1-position.y/height*2)
                if (dist < 0) {
                    idx = -1
                    wi = Qt.vector3d(0,0,0)
                } else {
                    idx = hit.vertex1Index
                    wi = Qt.vector3d(hit.worldIntersection.x, hit.worldIntersection.y, hit.worldIntersection.z)
                }
                ctx.picking_result(idx, pos, camera.projectionMatrix, camera.viewMatrix, puzzleTransform.matrix, wi)
            }
        }
        Entity {
            DirectionalLight {
                id: directional
                worldDirection: Qt.vector3d(0.0, 0.0, -1.0).normalized();
                color: "#ffffff"
                intensity: 0.5
            }
            Transform {
                id: lightpostransform
                //translation: Qt.vector3d(0.0, 50.0, 60.0)
            }
            components: [lightpostransform, directional]
        }
        Entity {
            id: puzzle_entity
            components: [puzzleTransform, mousehandler, screenraycaster]
            Component.onCompleted: ctx.set_puzzle_entity(puzzle_entity)
        }
    }
}

