//  Copyright © 2019  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQml 2.0
import QtQuick.Scene3D 2.0
import Qt3D.Core 2.0
import Qt3D.Render 2.11
import Qt3D.Logic 2.0

Scene3D {
    property RenderCaptureReply rendercapturereply: null
    property var size: Qt.size(1024,1024)
    
    focus: true
    aspects: ["render", "logic"]
    cameraAspectRatioMode: Scene3D.AutomaticAspectRatio
    
    function request_capture() {
        rendercapturereply = renderCapture.requestCapture()
    }
    
    Entity {
        components: [
            RenderSettings {
                activeFrameGraph: RenderSurfaceSelector {
                    surface: ctx.os_surface(8)
                    externalRenderTargetSize: size
                    
                    RenderTargetSelector {
                        target: RenderTarget {
                            attachments: [
                                RenderTargetOutput {
                                    attachmentPoint: RenderTargetOutput.Color0
                                    texture: Texture2DMultisample {
                                        width: size.width
                                        height: size.height
                                        format: Texture.RGBA8_UNorm
                                        samples: 8
                                        //minificationFilter: Texture.Linear
                                        //magnificationFilter: Texture.Linear
                                    }
                                },
                                RenderTargetOutput {
                                    attachmentPoint: RenderTargetOutput.Depth
                                    texture: Texture2DMultisample {
                                        width: size.width
                                        height: size.height
                                        format: Texture.DepthFormat
                                        samples: 8
                                        //minificationFilter: Texture.Linear
                                        //magnificationFilter: Texture.Linear
                                        //comparisonFunction: Texture.CompareLessEqual
                                        //comparisonMode: Texture.CompareRefToTexture
                                    }
                                }
                            ]
                            
                            CameraSelector {
                                camera: camera
                                Viewport {
                                    ClearBuffers {
                                        clearColor: Qt.rgba(0.,0.,0.,0.)
                                        buffers: ClearBuffers.ColorDepthBuffer
                            }   }   }
                        }
                        RenderCapture {
                            id: renderCapture
                        }
                }   }
            }
        ]
        
        FrameAction {
            onTriggered: {
                ctx.frame_triggered()
            }
        }
        Camera {
            id: camera
            projectionType: CameraLens.PerspectiveProjection
            fieldOfView: 44.1445/(ctx.zoom+0.269595) + 0.559741
            nearPlane : 0.1
            farPlane : 150.0
            position: Qt.vector3d( 0.0, 0.0, 4*ctx.bounding_sphere_radius)
            upVector: Qt.vector3d( 0.0, 1.0, 0.0 )
            viewCenter: Qt.vector3d( 0.0, 0.0, 0.0 )
        }
        Transform {
            id: puzzleTransform
            matrix: {
                var m = Qt.matrix4x4();
                m.rotate(ctx.rotationy, Qt.vector3d(1, 0, 0));
                m.rotate(ctx.rotationx, Qt.vector3d(0, 1, 0));
                return m;
        }   }
        Entity {
            DirectionalLight {
                id: directional
                worldDirection: Qt.vector3d(0.0, 0.0, -1.0).normalized();
                color: "#ffffff"
                intensity: 0.5
            }
            Transform {
                id: lightpostransform
                //translation: Qt.vector3d(0.0, 50.0, 60.0)
            }
            components: [lightpostransform, directional]
        }
        Entity {
            id: puzzle_entity
            components: [puzzleTransform]
            Component.onCompleted: ctx.set_puzzle_entity(puzzle_entity)
        }
    }
}

