//  Copyright © 2015-2017  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.7
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.1
import '.' as Pybik

Pybik.PreferencesPage {
        id: root
        title: ctx.tr("Settings")
        
        ItemDelegate {
            width: parent.width
            text: ctx.tr('Theme')
            onClicked: root.StackView.view.push(Qt.resolvedUrl("SettingsTheme.qml"))
        }
        Rectangle { height: 1; width: parent.width; color: palette.shadow }
        ItemDelegate {
            width: parent.width
            text: ctx.tr("Animation Speed:")
            down: false
        }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            contentItem: Slider {
                id: slider_animspeed
                width: parent.width
                orientation: Qt.Horizontal
                onValueChanged: ctx.set_settings("draw.speed", value)
                Component.onCompleted: {
                    // this values need to be set in correct order, so that
                    // no spurious valueChanged signal emitted. Changing
                    // from/to will emit such a signal if value
                    // is not in the new range.
                    var range = ctx.get_settings("draw.speed_range")
                    to = range[1] // initial 1.0
                    value = ctx.get_settings("draw.speed") // initial 0.0
                    from = range[0] // initial 0.0
            }   }
        }
        Rectangle { height: 1; width: parent.width; color: palette.shadow }
        SwitchDelegate {
            id: checkbox_mirrorfaces
            width: parent.width
            text: ctx.tr("Mirror Faces")
            checked: ctx.get_settings("draw.mirror_faces")
            onCheckedChanged: ctx.set_settings("draw.mirror_faces", checked)
        }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            contentItem: Item {
                implicitHeight: spinbox_mirrorfaces.implicitHeight
                Label {
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    text: ctx.tr("Mirror Distance:")
                }
                SpinBox {
                    id: spinbox_mirrorfaces
                    enabled: checkbox_mirrorfaces.checked
                    anchors.right: parent.right
                    stepSize: 1
                    property var mirrorfaces_range: ctx.get_settings("draw.mirror_distance_range")
                    from: mirrorfaces_range[0] * 10
                    to: mirrorfaces_range[1] * 10
                    value: ctx.get_settings("draw.mirror_distance") * 10
                    onValueChanged: ctx.set_settings("draw.mirror_distance", value / 10)
                    validator: DoubleValidator {
                        bottom: Math.min(spinbox_mirrorfaces.from, spinbox_mirrorfaces.to)
                        top:    Math.max(spinbox_mirrorfaces.from, spinbox_mirrorfaces.to)
                    }
                    textFromValue: function(value, locale)  { return Number(value / 10).toLocaleString(locale, 'f', 1) }
                    valueFromText: function(text, locale)   { return Number.fromLocaleString(locale, text) * 10 }
            }   }
        }
        Rectangle { height: 1; width: parent.width; color: palette.shadow }
        ItemDelegate {
            width: parent.width
            text: ctx.tr("Quality:")
            down: false
        }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            contentItem: Pybik.SimpleList {
                id: list_shader
                width: parent.width
                model: ctx.shadermodel
                onCurrentIndexChanged: ctx.set_settings("draw.shader", currentIndex)
                onModelChanged: currentIndex = ctx.get_settings("draw.shader")
        }   }
        ItemDelegate {
            width: parent.width
            text: ctx.tr("Antialiasing:")
            down: false
        }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            contentItem: Pybik.SimpleList {
                id: list_samples
                width: parent.width
                model: ctx.samplemodel
                onCurrentIndexChanged: ctx.set_settings("draw.samples", currentIndex)
                onModelChanged: currentIndex = ctx.sampleindex
        }   }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            visible: (ctx.activesamples > 1) && (ctx.activesamples != (1<<list_samples.currentIndex))
            contentItem: Label {
                id: label_needs_restarted
                width: parent.width
                //visible: false
                text: ctx.tr('The program needs to be restarted for the changes to take effect.')
                textFormat: Text.PlainText
                color: "#aa0000"
                wrapMode: Text.WordWrap
        }   }
        Control {
            width: parent.width
            padding: checkbox_mirrorfaces.padding
            contentItem: Button {
                width: parent.width
                text: ctx.tr('Reset Settings')
                onClicked: {
                    slider_animspeed.value = ctx.get_settings("draw.speed_default"); ctx.del_settings("draw.speed")
                    checkbox_mirrorfaces.checked = ctx.get_settings("draw.mirror_faces_default"); ctx.del_settings("draw.mirror_faces")
                    list_samples.currentIndex = ctx.get_settings("draw.samples_default"); ctx.del_settings("draw.samples")
                    list_shader.currentIndex = ctx.get_settings("draw.shader_default"); ctx.del_settings("draw.shader")
            }   }
        }
}

