//  Copyright © 2015-2017  B. Clausius <barcc@gmx.de>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import QtQuick 2.7
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.3
import '.' as Pybik

Pybik.PreferencesPage {
        title: ctx.tr("Theme")
        
        Connections {
            target: ctx
            onPrefupdateface: listview_faces.modelitem = ctx.facesmodelitem(listview_faces.currentkey)
        }
        
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Pybik.SimpleList {
                id: listview_faces
                width: parent.width
                model: ctx.facesmodel
                property var currentkey: model[currentIndex].key
                property var modelitem: ctx.facesmodelitem(currentkey)
                onCurrentIndexChanged: {
                    currentkey = model[currentIndex].key
                    modelitem = ctx.facesmodelitem(currentkey)
                }
        }   }
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Item {
                implicitHeight: button_color.implicitHeight
                Label {
                    id: button_color_text
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    text: ctx.tr("Color:")
                }
                Pybik.ColorButton {
                    id: button_color
                    x: button_color_text.width + 12
                    anchors.right: parent.right
                    Binding on color { value: listview_faces.modelitem.color }
                    onColorChanged: ctx.set_settings_arg("theme.faces.{}.color", listview_faces.currentkey, color.toString())
        }   }   }
        ItemDelegate {
            width: parent.width
            text: ctx.tr("Image File:")
            down: false
        }
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Item {
                implicitHeight: imageselector.height
                Pybik.ImageSelector {
                    id: imageselector
                    anchors.horizontalCenter: parent.horizontalCenter
                    z: 1
                
                    selectedvalue: listview_faces.modelitem.image
                    currentfolder: listview_faces.modelitem.folder
                    onImage_selected: {
                        ctx.set_settings_arg("theme.faces.{}.image", listview_faces.currentkey, name)
                        ctx.prefupdateface()
                    }
                    onImagefile_selected: {
                        ctx.set_settings_arg_url("theme.faces.{}.image", listview_faces.currentkey, url)
                        ctx.prefupdateface()
                   }
        }   }   }
        ButtonGroup { id: imagemodeGroup }
        RadioDelegate {
            id: radiobutton_tiled
            width: parent.width
            ButtonGroup.group: imagemodeGroup
            text: ctx.tr("Tiled")
            Binding on checked { value: listview_faces.modelitem.imagemode == "tiled" }
            onCheckedChanged: {
                if (checked)  ctx.set_settings_arg("theme.faces.{}.mode_nick", listview_faces.currentkey, "tiled")
            }
        }
        RadioDelegate {
            id: radiobutton_mosaic
            width: parent.width
            ButtonGroup.group: imagemodeGroup
            text: ctx.tr("Mosaic")
            Binding on checked { value: listview_faces.modelitem.imagemode == "mosaic" }
            onCheckedChanged: {
                if (checked)  ctx.set_settings_arg("theme.faces.{}.mode_nick", listview_faces.currentkey, "mosaic")
            }
        }
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Button {
                id: button_image_reset
                width: parent.width
                text: ctx.tr("Reset Face")
                onClicked: {
                    ctx.del_settings_arg("theme.faces.{}.color", listview_faces.currentkey)
                    ctx.del_settings_arg("theme.faces.{}.image", listview_faces.currentkey)
                    ctx.del_settings_arg("theme.faces.{}.mode", listview_faces.currentkey)
                    ctx.prefupdateface()
                }
        }   }
        Rectangle { height: 1; width: parent.width; color: palette.shadow }
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Item {
                implicitHeight: button_color.implicitHeight
                Label {
                    id: button_bgcolor_text
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    text: ctx.tr("Background:")
                }
                Pybik.ColorButton {
                    id: button_bgcolor
                    x: button_bgcolor_text.width + 12
                    anchors.right: parent.right
                    color: ctx.get_settings("theme.bgcolor")
                    onColorChanged: ctx.set_settings("theme.bgcolor", color.toString())
        }   }   }
        Control {
            width: parent.width
            padding: radiobutton_tiled.padding
            contentItem: Button {
                id: button_bgcolor_reset
                width: parent.width
                text: ctx.tr("Reset Background")
                onClicked: { button_bgcolor.color = ctx.get_settings("theme.bgcolor_default"); ctx.del_settings("theme.bgcolor")}
        }   }
}

