# -*- coding: utf-8 -*-
# cython: profile=False

#  Copyright © 2015-2019  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=C0326,W0614
# although this file is compiled with Python3 syntax, Cython needs at least division from __future__
from __future__ import print_function, division

# This line makes cython happy
global __name__, __package__    # pylint: disable=W0604
#px/__compiled = True
__compiled = False

import sys, os
from warnings import warn_explicit, warn

from pybiklib import config as config_
from pybiklib.settings import settings

#px+from [[_qtexec_VARIANT]] cimport *
#pxd+from qt cimport *
#pxd+from qtq cimport *
#pxd+from qt_impl cimport *
#px+cimport qt as Qt

#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxh>#include "Python.h"
#pxm>IF_END
#pxh>#include <QtCore/QtDebug>
#pxh>
#pxh>#include <QtCore/QObject>
#pxh>#include <QtCore/QString>
#pxh>#include <QtCore/QVariant>
#pxh>#include <QtGui/QImage>
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxh>#include <QtGui/QPainter>
#pxh>#include <QtGui/QDesktopServices>
#pxh>#include <QtGui/QStandardItemModel>
#pxh>#include <QtQuickWidgets/QQuickWidget>
#pxh>#include <QtWidgets/QMainWindow>
#pxh>#include <QtWidgets/QLineEdit>
#pxh>#include <QtWidgets/QLabel>
#pxh>#include <QtWidgets/QPushButton>
#pxh>#include <QtWidgets/QTreeView>
#pxh>#include <QtWidgets/QDialog>
#pxh>#include <QtWidgets/QListWidget>
#pxh>#include <QtWidgets/QListWidgetItem>
#pxh>#include <QtWidgets/QWidget>
#pxm>IF '[[QTVARIANT]]' == 'qtq'
#pxh>#include <QtQuick/QQuickItem>
#pxh>#include <QtQuick/QQuickView>
#pxm>IF_END
#pxh>
#pxc>#include "[[_qt_VARIANT]]_moc.h"
#pxc>

#pxm>IF '[[QTVARIANT]]' == 'qtw'
#px+from qtui_impl cimport UIContainer, gettext_translate
#px+cdef UIContainer qtui
#pxm>IF_END

#pxd>cdef extern from "[[_qt_VARIANT]]_moc.h":#
#pxm>DINDENT

#px/cdef enum: #
if True:
    DEBUG_MSG = 0x0002
    DEBUG_MSGGL = 0x0004
    DEBUG_MSGEXT = 0x0020
    DEBUG_DRAW = 0x0040
    DEBUG_FPS = 0x0080
    DEBUG_VFPS = 0x0100
    DEBUG_QML = 0x0200
    DEBUG_GLDEBUG = 0x0400
    DEBUG_SIM = 0x1000
    DEBUG_NOCONTROLS = 0x2000
#px+cdef extern long debug
#pxc>long debug;
debug = 0
    
#px:
# fake some names to be importable
Signal = lambda *args: None
QObject = QMainWindow = QLineEdit = QQuickWidget = QDialog = QImage = object
Qt = type('Qt', (), {'__getattr__': lambda self,attr: 0})()
sizeof = lambda obj: 0
#px.

def set_debug_flags(module):
    global debug
    if module.DEBUG_MSG:   debug |= DEBUG_MSG
    if module.DEBUG_MSGGL: debug |= DEBUG_MSGGL
    if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT
    if module.DEBUG_DRAW: debug |= DEBUG_DRAW
    if module.DEBUG_FPS: debug |= DEBUG_FPS
    if module.DEBUG_VFPS: debug |= DEBUG_VFPS
    if module.DEBUG_QML: debug |= DEBUG_QML
    if module.DEBUG_GLDEBUG: debug |= DEBUG_GLDEBUG
    if module.DEBUG_SIM: debug |= DEBUG_SIM
    if module.DEBUG_NOCONTROLS: debug |= DEBUG_NOCONTROLS
    if module.DEBUG_SHADERGL3: debug |= DEBUG_SHADERGL3
    if module.DEBUG_SHADERGL2: debug |= DEBUG_SHADERGL2
    if module.DEBUG_SHADERES2: debug |= DEBUG_SHADERES2
    
#pxm>IF '[[ANDROID]]' == 'android'
#px+cdef extern from "<android/log.h>":#
    #px+void __android_log_write(int, const char*, const char*) nogil
    #px+enum: ANDROID_LOG_DEBUG
# only use this function to replace sys.stout and sys.stderr
# from c++-code use qDebug, qInfo, qWarning, qCritical, qFatal
#px+def alog(text):
    #px+__android_log_write(ANDROID_LOG_DEBUG, 'pybik/py', <char*>text)
#pxm>IF_END

#px/cdef enum:
if True:
    SAFE_MODIFIER_MASK = Qt.ShiftModifier | Qt.ControlModifier | Qt.KeypadModifier
#pxm>IF '[[QTVARIANT]]' == 'qtq'
IGNORE_KEYS = [Qt.Key_Shift, Qt.Key_Control, Qt.Key_Meta, Qt.Key_Alt, Qt.Key_AltGr,
               Qt.Key_CapsLock, Qt.Key_NumLock, Qt.Key_ScrollLock]
#pxm>IF_END
class QtKeys:
    Key_Period = Qt.Key_Period
    Key_Escape = Qt.Key_Escape
    Key_Right = Qt.Key_Right
    Key_Left = Qt.Key_Left
    Key_Up = Qt.Key_Up
    Key_Down = Qt.Key_Down
    ShiftModifier = Qt.ShiftModifier
    ControlModifier = Qt.ControlModifier
    LeftButton = Qt.LeftButton
    RightButton = Qt.RightButton


#### Data ####

#px+ctypedef QObject *pQObject
#px+ctypedef QOffscreenSurface* pQOffscreenSurface
#px+DEF maxproxy = 21

#px/cdef struct UIData:
class uidata:  pass
    #px+QFileSystemWatcher* filesystemwatcher
    #px+QTimer *settings_timer
    #px+QSurfaceFormat default_format
    #px+QElapsedTimer *fps_monotonic_time
    #px+int fps_count
    
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+MainView *mainwindow
    #px+int modelselect_currentpage
    #px+QStandardItemModel *treestore
    #px+MoveEdit *move_edit
    #px+QLabel *statuslabel
    #px+DrawingArea *drawingarea
    #px+PreferencesDialog *preferences
    #px+QDialog *help
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    #px+cbool model_pagevisible
    #px+QQmlComponent *component
    #px+QQuickWindow *mainwindow
    #pxm>IF_END
    #px+QQmlEngine *engine
    #px+QObject *ctx
#px+cdef UIData uidata

class PyData:
    filesystemwatcher_path = None
    filesystemwatcher_handlers = {}
    singleshot_handler = None
    singleshot_queue = []
    app = None
    screenshot_filename = None
    modelselect = None
pydata = PyData()

#pxm-FUNC P
def init_module():
    if debug & DEBUG_MSGEXT:
        print('init module:', __name__)
        print('  from package:', __package__)
        print('  compiled:', __compiled)
        
    uidata.filesystemwatcher = NULL
    uidata.fps_monotonic_time = NULL
    if debug & DEBUG_FPS:
        #px+uidata.fps_monotonic_time = new QElapsedTimer()
        uidata.fps_monotonic_time.start()
    uidata.fps_count = 0
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    uidata.help = NULL
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    uidata.model_pagevisible = False
    uidata.component = NULL
    #pxm>IF_END
    uidata.mainwindow = NULL
    

#### Helpers ####

#pxm-FUNC P
def _q2list(ql:'const QVariantList &')->'':
    return [fromvariant(ql.at(i)) for i in range(ql.size())]
#pxm-FUNC P
def _qs2list(ql:'const QStringList &')->'':
    return [q2str(ql[i]) for i in range(ql.size())]
#pxm-FUNC P
def _q2dict(qm:'const QVariantMap &')->'':
    #px+cdef QStringList qkeys = qm.keys()
    #px+cdef QVariantList qvals = qm.values()
    return {k:v for k,v in zip(_qs2list(qkeys), _q2list(qvals))}
    
#pxm-FUNC P
def fromvariant(qvalue:'const QVariant &')->'':
    #px+cdef const char* typename = qvalue.typeName()
    if typename == NULL:
        return None
    if typename == b'bool':
        return qvalue.value[cbool]()
    elif typename == b'int':
        return qvalue.value[int]()
    elif typename == b'double':
        return qvalue.value[double]()
    elif typename == b'QString':
        return q2str(qvalue.value[QString]())
    elif typename == b'void*':
        res = qvalue.value[int]()
        return None if res == 0 else '<void*>({})'.format(res)
    elif qvalue.canConvert[QVariantMap]():
        return _q2dict(qvalue.value[QVariantMap]())
    elif qvalue.canConvert[QVariantList]():
        return _q2list(qvalue.value[QVariantList]())
    elif qvalue.canConvert[pQObject]():
        return 'QObject({})'.format(q2str(qvalue.value[pQObject]().objectName()))
    else:
        warn(RuntimeWarning('unknown typename %s' % typename[:]))
        return None

#pxm-FUNC P
def urltolocalfile(url:"QUrl")->tuple:
    if url.isLocalFile():
        return True, q2str(url.toLocalFile())
    else:
        return False, q2str(url.toString())
        
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#px/cdef QString gettext_translate "gettext_translate"(const char *text, const char *disambiguation) with gil:
def gettext_translate(text, disambiguation):
    pytext = _(text.decode('utf-8'))
    return str2q(pytext)
#pxm>IF_END
    
    
#### Qt3d ####

# vec3 pos, vec3 color, vec2 texCoord, vec3 normal
vertexbuffer_stride = (3+3+2+3+3) * sizeof(float)

#### Qt3d ####

#pxm-FUNC P
def set_atlas_data(i:int, image:'Image', x:int)->'cpdef':
    set_atlas_image(i, image.image, x)
    
def set_background_color(bgcolor):
    ctx_set(b'scene_bgcolor', bgcolor)

def set_shaders(shgl3vert, shgl3frag, shes2vert, shes2frag):
    set_shaders_impl(str2q(shgl3vert), str2q(shgl3frag), str2q(shes2vert), str2q(shes2frag))
    
def set_rotation_xy(x, y):
    x %= 360
    if y < -120:     y = -120
    elif y > 120:    y = 120
    ctx_set(b'rotationx', x)
    ctx_set(b'rotationy', y)
    return x, y
    
def set_frustum(bounding_sphere_radius, zoom):
    if bounding_sphere_radius > 0:
        ctx_set(b'bounding_sphere_radius', bounding_sphere_radius)
    ctx_set(b'zoom', zoom)
    
def set_transformations(blocks):
    for i, b, unused in blocks:
        set_entity_matrix(i, b)
    
#pxm-FUNC P
def set_animation_start(blocks, axisx:float, axisy:float, axisz:float)->'cpdef':
    blocks = list(blocks)
    set_animation_axis(len(blocks), axisx, axisy, axisz)
    for _i, (i, unused) in enumerate(blocks):
        set_animation_block(_i, i)
    
#pxm-FUNC P
def set_animation_next(angle:float)->'cpdef':
    set_animation_angle(-angle)
    
def set_gldata(nblocks, vertexdata, vertexsize, indexdata, cnts_block, transformations):
    assert vertexsize*sizeof(float) == vertexbuffer_stride, (vertexsize, sizeof(float), vertexbuffer_stride)
    assert nblocks == len(cnts_block)
    #px+cdef int nverts = len(vertexdata) // vertexbuffer_stride
    set_entity_count(QByteArray(vertexdata, len(vertexdata)), QByteArray(indexdata, len(indexdata)),
                    nverts, nblocks)
    
    offs = 0
    for i, cnt_block in enumerate(cnts_block):
        set_entity(i, offs*2, cnt_block)
        offs += cnt_block
        assert offs*2 <= len(indexdata)
    
    #px+cdef QMatrix4x4 mat
    #px+cdef int b, x, y
    set_transformation_count(len(transformations))
    for b, t in enumerate(transformations):
        for y in range(4):
            for x in range(4):
                mat.data()[y*4+x] = float(t[y][x])
        set_transformation(b, mat)
        
    
#### print gl info ####

#TODO: use or remove it
#pxm>IF False
#pxm-FUNC P nogil
def print_surface_info():
    #px+cdef QOpenGLContext *glcontext
    glcontext = currentContext()
    glformat = glcontext.format()
    glrformat = defaultFormat() # requested format
    gldformat = uidata.default_format
    qInfo('Surface format (requested (), default []):')
    _pfmt('red', glformat.redBufferSize(), glrformat.redBufferSize(), gldformat.redBufferSize())
    _pfmt('green', glformat.greenBufferSize(), glrformat.greenBufferSize(),  gldformat.greenBufferSize())
    _pfmt('blue', glformat.blueBufferSize(), glrformat.blueBufferSize(), gldformat.blueBufferSize())
    _pfmt('alpha', glformat.alphaBufferSize(), glrformat.alphaBufferSize(), gldformat.alphaBufferSize())
    _pfmt('depth', glformat.depthBufferSize(), glrformat.depthBufferSize(), gldformat.depthBufferSize())
    _pfmt('options', glformat.options(), glrformat.options(), gldformat.options())
    _pfmt('profile', glformat.profile(), glrformat.profile(), gldformat.profile())
    _pfmt('renderableType', glformat.renderableType(), glrformat.renderableType(), gldformat.renderableType())
    _pfmt('samples', glformat.samples(), glrformat.samples(), gldformat.samples())
    _pfmt('stencil', glformat.stencilBufferSize(), glrformat.stencilBufferSize(), gldformat.stencilBufferSize())
    _pfmt('stereo', glformat.stereo(), glrformat.stereo(), gldformat.stereo())
    _pfmt('swapBehavior', glformat.swapBehavior(), glrformat.swapBehavior(), gldformat.swapBehavior())
    _pfmt('swapInterval', glformat.swapInterval(), glrformat.swapInterval(), gldformat.swapInterval())
    _pfmt2('version', glformat.majorVersion(), glformat.minorVersion(),
                     glrformat.majorVersion(), glrformat.minorVersion(),
                     gldformat.majorVersion(), gldformat.minorVersion())
    qInfo('OpenGL context:')
    #px+cdef int i
    i = openGLModuleType()
    if i == LibGL:
        qInfo('  module type: libGL')
    elif i == LibGLES:
        qInfo('  module type: LibGLES')
    else:
        qInfo('  unknown module type: %d', i)
    i = glcontext.isOpenGLES()
    if glcontext.isOpenGLES():
        qInfo('  isOpenGLES: True')
    else:
        qInfo('  isOpenGLES: False')
    if glcontext.hasExtension(QByteArray(b"GL_KHR_debug")):
        qInfo('  hasExtension GL_KHR_debug: True')
    else:
        qInfo('  hasExtension GL_KHR_debug: False')
#pxm>IF_END
    

#### ui: communicate with render-thread  ####

#pxm-FUNC P
def set_atlas_size(width:int, height:int)->'cpdef':
    set_atlas_image_size(width, height)
    
def set_atlas_texture():
    set_atlas_texture_impl()
    
#pxm-FUNC P
def set_default_surface_format():
    #px+cdef QSurfaceFormat glformat
    #px+cdef int samples
    glformat.setDepthBufferSize(8)
    samples = settings['draw.samples']
    if samples > 0:
        glformat.setSamples(2**samples)
    if debug & DEBUG_VFPS:
        glformat.setSwapInterval(0)
    if debug & DEBUG_GLDEBUG:
        glformat.setOption(DebugContext)
    uidata.default_format = defaultFormat()
    setDefaultFormat(glformat)
    
    
#### ui: communicate with qml ####

#pxm-QD_CPPCLASS
class SectionNameIndexItem (QObject):
    pass
    #pxm>Q_PROPERTY prop_section = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_name = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_index = 'int MEMBER'
    #pxh>
    #pxm-FUNC HD
    def __init__(section:str, name:str, index:int):  pass
    #pxh>{ m_section = section; m_name = name; m_index = index; }
#pxm>CPPCLASS_END

#pxm-QD_CPPCLASS
class TextKeyItem (QObject):
    pass
    #pxm>Q_PROPERTY prop_text = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_key = 'QString MEMBER NOTIFY'
    #pxh>
    #pxm-FUNC HD nogil
    def __init__(text:str, key:str):  pass
    #pxh>{ m_text = text; m_key = key; }
#pxm>CPPCLASS_END

#pxm-QD_CPPCLASS
class FacePrefsItem (QObject):
    pass
    #pxm>Q_PROPERTY prop_color = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_folder = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_image = 'QString MEMBER NOTIFY'
    #pxm>Q_PROPERTY prop_imagemode = 'QString MEMBER NOTIFY'
    #pxh>
    #pxm-FUNC HD nogil
    def __init__(color:str, folder:str, image:str, imagemode:str):  pass
    #pxh>{ m_color = color; m_folder = folder; m_image = image; m_imagemode = imagemode; }
#pxm>CPPCLASS_END

#pxc>#include <math.h>
#pxm-FUNC CHD nogil
def _get_cursor_angle(position:'const QPointF *', projectionmatrix:1, viewmatrix:1, puzzlematrix:'const QMatrix4x4 *', worldintersection:1, d:'const QVector3D *')->float: pass
#pxc>{
#pxc>    auto v = *projectionmatrix * *viewmatrix * (*worldintersection + (*puzzlematrix * *d));
#pxc>    return atan2(position->x()-v.x(), position->y()-v.y()) * 180.0 / M_PI;
#pxc>}

#px/cdef class PickingResult:
class PickingResult:
    #px+cdef const QPointF *position
    #px+cdef const QMatrix4x4 *projectionmatrix
    #px+cdef const QMatrix4x4 *viewmatrix
    #px+cdef const QMatrix4x4 *puzzlematrix
    #px+cdef const QVector3D *worldintersection
    
    def get_cursor_angle(self, d):
        #px+cdef QVector3D vd = QVector3D(d[0], d[1], d[2])
        return _get_cursor_angle(self.position,
                        self.projectionmatrix, self.viewmatrix, self.puzzlematrix,
                        #px+self.worldintersection, &vd
                    )
        
#pxm-QD_CPPCLASS
class ContextProperty (QObject):
    #pxd>pass
    #pxm-FUNC PH nogil
    def slot_inspect(self, msg:str, val:'const QVariant &'):
        if val.canConvert[pQObject]():
            dump_info(msg.toUtf8().data(), val.value[pQObject]())
        else:
            with gil:
                if val.isValid():
                    print('{}: {}: {}'.format(q2str(msg), val.typeName()[:], fromvariant(val)))
                else:
                    print('{}: not valid: {}'.format(q2str(msg), fromvariant(val)))
    #pxm-FUNC PH with gil
    def slot_tr(self, string:str)->str:
        try:
            return str2q(_(q2str(string)))
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_config(self, attr:str)->str:
        try:
            return str2q(getattr(config_, q2str(attr)))
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_config_tr(self, attr:str)->str:
        try:
            return str2q(_(getattr(config_, q2str(attr))))
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_get_settings(self, key:str)->'QVariant':
        try:
            return tovariant(settings[q2str(key)])
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_set_settings(self, key:str, value:'const QVariant &'):
        try:
            settings[q2str(key)] = fromvariant(value)
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_set_settings_arg(self, key:str, arg:str, value:'const QVariant &'):
        try:
            if not arg.isEmpty():
                settings[q2str(key).format(q2str(arg))] = fromvariant(value)
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_set_settings_arg_url(self, key:str, arg:str, qurl:'const QUrl &'):
        try:
            if arg.isEmpty():  return
            url = q2str(qurl.toString())
            if not url.startswith('file:///'):  return
            settings[q2str(key).format(q2str(arg))] = url[7:]
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_del_settings(self, key:str):
        try:
            del settings[q2str(key)]
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH with gil
    def slot_del_settings_arg(self, key:str, arg:str):
        try:
            if not arg.isEmpty():
                del settings[q2str(key).format(q2str(arg))]
        except:  sys.excepthook(*sys.exc_info())
    #pxm-FUNC PH nogil
    def slot_set_puzzle_entity(self, entity:'QObject *'):
        set_puzzle_entity_impl(entity)
    #pxm-FUNC PH with gil
    def slot_animation_step(self, angle:'qreal'):
        set_animation_next(angle)
    #pxm-FUNC PH with gil
    def slot_animation_stopped(self):
        pydata.app.on_animation_ending()
    #pxm-FUNC PH with gil
    def slot_plugin_activated(self, index:int):
        pydata.app.on_plugin_activated(index)
    #pxm-FUNC PH with gil
    def slot_editing_finished(self, code:str, pos:int):
        pydata.app.on_edit_finished(q2str(code), pos)
    #pxm-FUNC PH with gil
    def slot_action(self, string:str):
        getattr(pydata.app, 'on_action_%s_triggered' % q2str(string))()
        
    #pxm-FUNC PH with gil
    def slot_key_pressed(self, key:int, modifiers:int, fromdrawingarea:bool)->bool:
        return pydata.app.on_key_pressed(key, modifiers & SAFE_MODIFIER_MASK, fromdrawingarea)
    #pxm-FUNC PH nogil
    def slot_hover_enabled(self)->bool:
        return debug & DEBUG_DRAW
    #pxm-FUNC PH with gil
    def slot_mouse_pressed(self, modifiers:int, button:int, x:int, y:int):
        pydata.app.on_mouse_pressed(modifiers, button, x, y)
    #pxm-FUNC PH with gil
    def slot_drawingarea_mouse_released(self, x:int, y:int):
        pydata.app.on_mouse_released(x, y)
    #pxm-FUNC PH with gil
    def slot_drawingarea_mouse_moved(self, buttons:int, x:int, y:int):
        pydata.app.on_mouse_moved(buttons, x, y)
    #pxm-FUNC PH with gil
    def slot_drawingarea_zoom(self, delta:'qreal'):
        pydata.app.on_mouse_zoom(delta)
    #pxm-FUNC PH with gil
    def slot_drag_dropped_color(self, color:str):
        pydata.app.on_mouse_drop_color(q2str(color))
    #pxm-FUNC PH with gil
    def slot_drag_dropped_url(self, url:str):
        pydata.app.on_mouse_drop_url(*urltolocalfile(QUrl(url)))
    #pxm-FUNC PH with gil
    def slot_picking_result(self, vertex1idx:int, position:'const QPointF &', projectionmatrix:1, viewmatrix:1, puzzlematrix:'const QMatrix4x4 &', worldintersection:'const QVector3D &'):
        pick = PickingResult()
        #px+pick.position = &position
        #px+pick.projectionmatrix = &projectionmatrix
        #px+pick.viewmatrix = &viewmatrix
        #px+pick.puzzlematrix = &puzzlematrix
        #px+pick.worldintersection = &worldintersection
        pydata.app.on_picking_result(vertex1idx, pick)
        
    #pxm-FUNC PH with gil
    def slot_frame_triggered(self):
        #px+cdef int fps
        if debug & DEBUG_FPS:
            uidata.fps_count += 1
            if not uidata.fps_monotonic_time.hasExpired(1000):
                return
            fps = 1000 * uidata.fps_count // uidata.fps_monotonic_time.restart()
            uidata.fps_count = 0
            uidata.ctx.setProperty(b'debugmsg', QVariant(fps))
    #pxm-FUNC PH nogil
    def slot_os_surface(self, samples:int)->'QVariant':
        #px+cdef QOffscreenSurface *surface = new QOffscreenSurface()
        #px+cdef QSurfaceFormat format = QSurfaceFormat()
        format.setDepthBufferSize(8)
        format.setSamples(samples)
        surface.setFormat(format)
        surface.create()
        #px+cdef QVariant var;
        var.setValue[pQOffscreenSurface](surface)
        return var
            
    #pxm-FUNC PH with gil
    def slot_dialog_text(self, string:str)->str:
        return str2q(pydata.app.on_dialog_text(q2str(string)))
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    #pxm-FUNC PH with gil
    def slot_model_start_selection(self):
        if pydata.modelselect is None:
            pydata.modelselect = pydata.app.on_action_selectmodel_triggered()
        uidata.model_pagevisible = True
    #pxm-FUNC PH nogil
    def slot_model_pagevisible(self)->bool:
        return uidata.model_pagevisible
    #pxm-FUNC PH with gil
    def slot_model_get_modeldata(self, pageindex:int)->'QVariant':
        items, index, title = pydata.modelselect.on_model_get_modeldata(pageindex)
        ctx_set(b'selectmodel_title', title)
        ctx_set(b'selectmodel_index', index)
        return create_stdmodel(items)
    #pxm-FUNC PH with gil
    def slot_model_selected(self, pageindex:int, index:int)->bool:
        mtype_size = pydata.modelselect.on_convert_to_mtype_size(pageindex, index)
        if mtype_size is None:
            return True
        pydata.app.on_load_other_game(*mtype_size)
        uidata.model_pagevisible = False
        return False
    #pxm-FUNC PH with gil
    def slot_model_back(self):
        if pydata.modelselect.on_model_back():
            return
        uidata.model_pagevisible = False
            
    #pxm-FUNC PH with gil
    def slot_movekey_item_changed(self):
        settings['draw.accels'] = ctx_get_stdmodel(b'movekey_model')
    #pxm-FUNC PH with gil
    def slot_movekey_item_added(self, index:int)->int:
        values = ctx_get_stdmodel(b'movekey_model')
        if 0 <= index < len(values):
            values.insert(index, ('', ''))
        else:
            index = len(values)
            values.append(('', ''))
        ctx_set_stdmodel(b'movekey_model', values)
        settings['draw.accels'] = values
        return index
    #pxm-FUNC PH with gil
    def slot_movekey_item_removed(self, index:int)->int:
        values = ctx_get_stdmodel(b'movekey_model')
        del values[index]
        ctx_set_stdmodel(b'movekey_model', values)
        settings['draw.accels'] = values
        return min(index, len(values)-1)
    #pxm-FUNC PH with gil
    def slot_movekey_items_reset(self):
        del settings['draw.accels']
        ctx_set_stdmodel(b'movekey_model', settings['draw.accels'])
    #pxm-FUNC PH with gil
    def slot_keytostring(self, key:int, modifiers:int)->str:
        if key in IGNORE_KEYS:
            return QString()
        modifiers &= SAFE_MODIFIER_MASK
        return QKeySequence(key|modifiers, 0,0,0).toString(PortableText)
    #pxm-FUNC PH with gil const
    def slot_facesmodelitem(self, qkey:str)->'QObject *':
        key = q2str(qkey)
        assert key
        color = settings['theme.faces',key,'color']
        folder, image = get_imagefile(key)
        imagemode = settings['theme.faces',key,'mode_nick']
        #px+return <QObject*>new FacePrefsItem(str2q(color), str2q(folder), str2q(image), str2q(imagemode))
    #pxm>IF_END
        
#pxh>
    #pxm:Q_SIGNAL
    prefupdateface = Signal()
    prefupdatebkgr = Signal()
    #px.
#pxh>
    #pxm:Q_PROPERTY
    prop_edittext1 = 'QString MEMBER NOTIFY'
    prop_edittext2 = 'QString MEMBER NOTIFY'
    prop_editposition = 'QString MEMBER NOTIFY'
    prop_toolbarstate = 'QVariantList MEMBER NOTIFY'
    prop_sidepanemodel = 'QList<QObject*> MEMBER NOTIFY'
    prop_debugmsg = 'QString MEMBER NOTIFY'
    prop_status = 'QString MEMBER NOTIFY'
    prop_animation_running = 'bool MEMBER NOTIFY'
    prop_animation_speed = 'float READ NOTIFY'
    prop_capture_filename = 'QString READ NOTIFY'
    prop_animation_angle = 'float MEMBER NOTIFY'
    prop_animation_maxangle = 'float MEMBER NOTIFY'
    prop_messagebox_visible = 'bool MEMBER NOTIFY'
    prop_messagetext = 'QString MEMBER NOTIFY'
    prop_selectmodel_title = 'QString MEMBER'
    prop_selectmodel_index = 'int MEMBER'
    prop_preferences_visible = 'bool MEMBER NOTIFY'
    prop_shadermodel = 'QList<QObject*> MEMBER NOTIFY'
    prop_samplemodel = 'QList<QObject*> MEMBER NOTIFY'
    prop_activesamples = 'int MEMBER NOTIFY'
    prop_sampleindex = 'int MEMBER'
    prop_movekey_model = 'QList<QObject*> MEMBER NOTIFY'
    prop_facesmodel = 'QList<QObject*> MEMBER NOTIFY'
    prop_imagemodel = 'QList<QObject*> MEMBER NOTIFY'
    prop_helpdialog_visible = 'bool MEMBER NOTIFY'
    prop_aboutdialog_visible = 'bool MEMBER NOTIFY'
    prop_scene_bgcolor = 'QString MEMBER NOTIFY'
    prop_zoom = 'float MEMBER NOTIFY'
    prop_bounding_sphere_radius = 'float MEMBER NOTIFY'
    prop_rotationx = 'float MEMBER NOTIFY'
    prop_rotationy = 'float MEMBER NOTIFY'
    #px.
    #pxh>
    #pxm-FUNC PH with gil const
    def get_animation_speed(self)->float:
        return settings['draw.speed']
    #pxm-FUNC PH with gil const
    def get_capture_filename(self)->'QString':
        return str2q(pydata.screenshot_filename) if pydata.screenshot_filename else QString()
        
#pxm>CPPCLASS_END
    
    
class QmlError (Warning): pass

# also used in beforeSynchronizing
#pxm-FUNC P nogil
def dump_info(msg:'const char *', qtobj:'const QObject *'):
    #px+cdef const QMetaObject *mo
    #px+cdef QMetaProperty p
    #px+cdef QMetaMethod m
    #px+cdef int i
    
    mo = qtobj.metaObject()
    
    qInfo('==== %s', msg);
    #qInfo('Object-Name: %s', q2charp(qtobj.objectName()))
    qInfo('Class-Name: %s', mo.className())
    if 0 < mo.propertyCount():
        qInfo('Properties: %d (%d inherited)', mo.propertyCount(), mo.propertyOffset())
        for i in range(mo.propertyCount()):
            p = mo.property(i)
            qInfo('  %d %s %s', i, p.typeName(), p.name())
    if 0 < mo.methodCount():
        qInfo('Methods: %d (%d inherited)', mo.methodCount(), mo.methodOffset())
        for i in range(mo.methodCount()):
            m = mo.method(i)
            qInfo('  %d %s %s', i, m.typeName(), m.methodSignature().data())
            
#pxm-FUNC P with gil
def print_warnings(errors:'const QmlErrorList &'):
    #px+cdef int i
    #px+cdef QQmlError error
    for i in range(errors.size()):
        error = errors.at(i)
        filename = q2str(error.url().toLocalFile())
        msg, line = q2str(error.description()), error.line()
        print('{2}:{3}:{4}: {0}'.format(msg, QmlError, filename, line, error.column()), file=sys.stderr)
        warn_explicit(msg, QmlError, filename, line, module=os.path.basename(filename))
        
def set_debug_text(text):
    ctx_set(b'debugmsg', text)
    
def set_edit_moves(code, pos):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    uidata.move_edit.setText(str2q(code))
    uidata.move_edit.setCursorPosition(pos)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    uidata.ctx.setProperty(b'edittext1', QVariant(str2q(code[:pos]).toHtmlEscaped()))
    uidata.ctx.setProperty(b'edittext2', QVariant(str2q(code[pos:]).toHtmlEscaped()))
    ctx_set(b'editposition', pos)
    #pxm>IF_END
    
def set_status_text(text):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    uidata.statuslabel.setText(str2q(text))
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    ctx_set(b'status', text)
    #pxm>IF_END
    
show_message_text = None
#pxm-FUNC P with gil
def show_message_cb():
    global show_message_text
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+cdef QMessageBox *dialog = new QMessageBox(uidata.mainwindow)
    dialog.setWindowTitle(str2q(_(config_.APPNAME)))
    dialog.setText(str2q(show_message_text))
    dialog.setIcon(QMessageBox_Information)
    dialog.setStandardButtons(QMessageBox_Close)
    dialog.exec()
    del dialog
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    ctx_set(b'messagetext', show_message_text)
    ctx_set(b'messagebox_visible', True)
    #pxm>IF_END
    show_message_text = None
def show_message(message):
    global show_message_text
    show_message_text = message
    singleShot(100, show_message_cb)
    
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxm-FUNC P
def _fill_subtree(parent:'QStandardItem *', tree):
    #px+cdef QStandardItem *item
    for transl, func, subtree in tree:
        #px+item = new QStandardItem(str2q(transl))
        if func is not None:
            item.setData(tovariant(func))
        parent.appendRow(item)
        _fill_subtree(item, subtree)
        
#pxm-FUNC P
def _fill_treestore(plugin_data):
    # remove old plugins
    uidata.treestore.clear()
    qtui.clear_sidepane()
    # fill treestore with plugins
    #px+cdef QStandardItem *item
    for transl, func, subtree in plugin_data:
        #px+item = new QStandardItem(str2q(transl))
        if func is not None:
            item.setData(tovariant(func))
        uidata.treestore.appendRow(item)
        _fill_subtree(item, subtree)
#pxm>IF_END
        
def fill_sidepane(plugin_data):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    _fill_treestore(plugin_data)
    # create widgets in the sidepane to display the plugins
    #px+cdef QPushButton *button
    #px+cdef QTreeView *treeview
    for i, [transl, func, subtree] in enumerate(plugin_data):
        button = qtui.create_sidepane_button(str2q(transl))
        treeview = qtui.create_sidepane_treeview(uidata.treestore, i)
        uidata.mainwindow.connect_sidepane(button, treeview)
        qtui.set_active_plugin_group(-1)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    pydata.plugin_data = plugin_data
    #pxm>IF_END
        
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxm-FUNC P
def _hide_row(treeview:int, index:'const QModelIndex &')->bool:
    funcidx = fromvariant(index.data(Qt.UserRole + 1))
    if not pydata.app.on_plugin_test_idx(funcidx):
        return True
    hide_all = funcidx is None
    rows = uidata.treestore.rowCount(index)
    for r in range(rows):
        hide = _hide_row(treeview, uidata.treestore.index(r, 0, index))
        qtui.set_row_hidden(treeview, r, index, hide)
        hide_all = hide_all and hide
    return hide_all
#pxm>IF_END
def update_sidepane():
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+cdef QModelIndex index
    for i in range(qtui.get_plugin_group_count()):
        index = qtui.root_index(i)
        qtui.hide_row(i, _hide_row(i, index))
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    def filter_plugin_data(tree):
        for section, func, subtree in tree:
            assert func is None
            for transl1, func, subtree in subtree:
                funcidx1 = -1 if func is None else func
                if pydata.app.on_plugin_test_idx(funcidx1):
                    subptree = []
                    for transl2, func, subtree in subtree:
                        assert not subtree, subtree
                        funcidx = -1 if func is None else func
                        if pydata.app.on_plugin_test_idx(funcidx):
                            subptree.append((section, ' • ' + transl2, funcidx))
                    if subptree or funcidx1 >= 0:
                        yield section, transl1, funcidx1
                        for section, name, funcidx in subptree:
                            yield section, name, funcidx
    ctx_set_ssi_model(b'sidepanemodel', filter_plugin_data(pydata.plugin_data))
    #pxm>IF_END
        
def set_toolbar_state(state):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    qtui.set_toolbar_state(state[0], state[1], state[2], state[3], state[4])
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    ctx_set(b'toolbarstate', state)
    #pxm>IF_END

def ctx_set(propname, value):
    uidata.ctx.setProperty(propname, tovariant(value))
    
#pxm>IF '[[QTVARIANT]]' == 'qtq'
#pxm-FUNC P
def create_ssi_model(values)->'QVariant':
    #px+cdef QObjectList qlist
    #qlist.reserve(len(values))
    for section, name, index, *unused in values:
        #px+qlist.append(<QObject*>new SectionNameIndexItem(str2q(section), str2q(name), index))
        pass
    #px+cdef QVariant items
    items.setValue[QObjectList](qlist)
    return items
    
def ctx_set_ssi_model(propname, values):
    uidata.ctx.setProperty(propname, create_ssi_model(values))
    
def ctx_get_stdmodel(propname):
    #px+cdef QVariantList qlist = uidata.ctx.property(propname).value[QVariantList]()
    #px+cdef QObject *qitem
    items = []
    for i in range(qlist.size()):
        qitem = qlist.at(i).value[pQObject]()
        items.append((fromvariant(qitem.property(b'text')), fromvariant(qitem.property(b'key'))))
    return items
#pxm>IF_END
    
#pxm-FUNC P
def create_stdmodel(values)->'QVariant':
    #px+cdef QObjectList qlist
    for text, key, *unused in values:
        #px/qlist.append(<QObject*>new TextKeyItem(str2q(text), str2q(key)))
        pass
    #px+cdef QVariant items
    items.setValue[QObjectList](qlist)
    return items
    
#pxm>IF '[[QTVARIANT]]' == 'qtq'
def ctx_set_stdmodel(propname, values):
    uidata.ctx.setProperty(propname, create_stdmodel(values))
#pxm>IF_END
    
def ctx_invoke(name):
    invokeMethod(uidata.ctx, name, Qt.DirectConnection)
    
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxm-QD_CPPCLASS
class MainView (QMainWindow):
    #pxm-FUNC HD nogil
    def __init__(): pass
        #pxh> : QMainWindow() {}
        
    #pxm-FUNC CHD nogil
    def connect_sidepane(self, button:'QPushButton *', treeview:'QTreeView *'):  pass
    #pxc>{
        #pxc>connect(button, SIGNAL(clicked()), this, SLOT(_on_button_sidepane_clicked()), Qt::DirectConnection);
        #pxc>connect(treeview, SIGNAL(activated(const QModelIndex &)), this, SLOT(_on_treeview_activated(const QModelIndex &)), Qt::DirectConnection);
    #pxc>}
    
    #pxm-FUNC PH with gil
    def resizeEvent(this, event:'QResizeEvent *'):
        settings['window.size'] = event.size().width(), event.size().height()
        pos = qtui.splitter_pos()
        if pos > 0:  # ignore the first resize event where sizes==[0,0]
            settings['window.divider'] = pos
        this.QWidget_resizeEvent(event)
    #pxm-FUNC PH with gil
    def closeEvent(this, event:'QCloseEvent *'):
        destroy_resources()
        try:
            pydata.app.on_closing()
        finally:
            this.QWidget_closeEvent(event)
    #pxm-FUNC PH with gil
    def keyPressEvent(this, event:'QKeyEvent *'):
        if not pydata.app.on_key_pressed(event.key(), int(event.modifiers()) & SAFE_MODIFIER_MASK, False):
            event.ignore()
            this.QWidget_keyPressEvent(event)
#pxh>
    #pxm:ACTION
    action_challenge = pydata
    action_new_solved = pydata
    action_preferences = pydata
    action_reset_rotation = pydata
    action_rewind = pydata
    action_previous = pydata
    action_stop = pydata
    action_play = pydata
    action_next = pydata
    action_forward = pydata
    action_mark_set = pydata
    action_mark_remove = pydata
    action_initial_state = pydata
    action_edit_cube = pydata
    #px.
    
    #pxm-FUNC PH with gil
    def slot_on_action_selectmodel_triggered(this):
        if pydata.modelselect is None:
            pydata.modelselect = pydata.app.on_action_selectmodel_triggered()
        uidata.modelselect_currentpage = 1
        items, index, title = pydata.modelselect.on_model_get_modeldata(1)
        qtui.fill_page(create_stdmodel(items), str2q(title))
        qtui.set_page(1)
    #pxm-FUNC PH with gil
    def slot_on_action_selectmodel_back_triggered(this):
        if pydata.modelselect.on_model_back():
            uidata.modelselect_currentpage -= 1
            items, index, title = pydata.modelselect.on_model_get_modeldata(uidata.modelselect_currentpage)
            qtui.fill_page(create_stdmodel(items), str2q(title))
            qtui.set_page(1)
        else:
            qtui.set_page(0)
    #pxm-FUNC PHD with gil
    def _listwidget_itemActivated(self, item:'QListWidgetItem *'):
        index = item.listWidget().currentRow()  #TODO: store index in item
        mtype_size = pydata.modelselect.on_convert_to_mtype_size(uidata.modelselect_currentpage, index)
        if mtype_size is None:
            uidata.modelselect_currentpage += 1
            items, index, title = pydata.modelselect.on_model_get_modeldata(uidata.modelselect_currentpage)
            qtui.fill_page(create_stdmodel(items), str2q(title))
        else:
            pydata.app.on_load_other_game(*mtype_size)
            qtui.set_page(0)
            while pydata.modelselect.on_model_back():
                pass
    #pxm-FUNC PH with gil
    def slot_on_listwidget_itemActivated(this, item:'QListWidgetItem *'):
        this._listwidget_itemActivated(item)
    #pxm-FUNC PH with gil
    def slot_on_listwidget_itemClicked(this, item:'QListWidgetItem *'):
        this._listwidget_itemActivated(item)
    #pxm-FUNC PH with gil
    def slot_on_action_quit_triggered(this):
        this.close()
    #pxm-FUNC PH with gil
    def slot_on_action_editbar_toggled(this, checked:bool):
        qtui.set_frame_editbar_visible(checked)
        settings['window.editbar'] = checked
    #pxm-FUNC PH with gil
    def slot_on_action_statusbar_toggled(this, checked:bool):
        qtui.set_statusbar_visible(checked)
        settings['window.statusbar'] = checked
    #pxm-FUNC PH with gil
    def slot_on_action_help_triggered(this):
        if uidata.help == NULL:
            help_text = pydata.app.on_dialog_text('help_text')
            uidata.help = qtui.setupUi_help(uidata.mainwindow, str2q(help_text))
        uidata.help.show()
    #pxm-FUNC PH with gil
    def slot_on_action_info_triggered(this):
        about = qtui.setupUi_about(uidata.mainwindow)
        
        qtui.fill_header(str2q(config_.APPICON_FILE), str2q(_(config_.APPNAME)),
                         str2q(config_.VERSION), str2q(_(config_.SHORT_DESCRIPTION)))
        qtui.fill_about_tab(str2q(config_.COPYRIGHT),
                            str2q(pydata.app.on_dialog_text('about_website')),
                            str2q(pydata.app.on_dialog_text('about_translators')))
        qtui.fill_contribute_tab(str2q(pydata.app.on_dialog_text('about_contribute')))
        license_full = pydata.app.on_dialog_text('about_license_full')
        if license_full:
            qtui.fill_license_tab(str2q(pydata.app.on_dialog_text('about_license_short')),
                                  False, str2q(license_full))
        else:
            qtui.fill_license_tab(str2q(pydata.app.on_dialog_text('about_license_short')),
                                  True, str2q(pydata.app.on_dialog_text('about_license_notfound')))
        about.exec()
        about.deleteLater()
    #pxm-FUNC PH with gil
    def slot_on_splitter_splitterMoved(self, pos:int, index:int):
        if index == 1:
            settings['window.divider'] = pos
            qtui.splitter_update_minimumsize()
    #pxm-FUNC PH with gil
    def slot_on_button_edit_clear_clicked(this):
        uidata.move_edit.setText(str2q(''))
        pydata.app.on_edit_finished(q2str(uidata.move_edit.text()), uidata.move_edit.cursorPosition())
    #pxm-FUNC PH with gil
    def slot_on_button_edit_exec_clicked(this):
        pydata.app.on_edit_finished(q2str(uidata.move_edit.text()), uidata.move_edit.cursorPosition())
    #pxm-FUNC PH with gil
    def slot_on_action_jump_to_editbar_triggered(this):
        uidata.move_edit.setFocus()
        
    #pxm-FUNC PH with gil
    def slot__on_button_sidepane_clicked(this):
        qtui.set_active_plugin_group_by_obj(this.sender())
    #pxm-FUNC PH with gil
    def slot__on_treeview_activated(this, index:'const QModelIndex &'):
        func_idx = fromvariant(index.data(Qt.UserRole + 1))
        if func_idx is None:
            return
        pydata.app.on_plugin_activated(func_idx)
#pxm>CPPCLASS_END


#pxm-QD_CPPCLASS
class MoveEdit (QLineEdit):
    #pxd>pass
    #pxm-FUNC CHD
    def __init__(this):  pass
        #pxc> : QLineEdit() {
        #pxc>connect(this, SIGNAL(returnPressed()), this, SLOT(on_returnpressed()), Qt::DirectConnection);
        #pxc>}
    #pxm-FUNC PH with gil
    def keyPressEvent(this, event:'QKeyEvent *'):
        if event.matches(MoveToNextWord):
            pos = pydata.app.on_edit_moves_nextword(q2str(this.text()), this.cursorPosition())
            this.setCursorPosition(pos)
        elif event.matches(MoveToPreviousWord):
            pos = pydata.app.on_edit_moves_prevword(q2str(this.text()), this.cursorPosition())
            this.setCursorPosition(pos)
        elif event.key() == Qt.Key_Right and event.modifiers() == Qt.AltModifier:
            pydata.app.on_edit_moves_swapnext(q2str(this.text()), this.cursorPosition())
        elif event.key() == Qt.Key_Left and event.modifiers() == Qt.AltModifier:
            pydata.app.on_edit_moves_swapprev(q2str(this.text()), this.cursorPosition())
        else:
            this.QLineEdit_keyPressEvent(event)
    #pxm-FUNC PH with gil const
    def slot_on_returnpressed(self):
        pydata.app.on_edit_finished(q2str(this.text()), this.cursorPosition())
#pxm>CPPCLASS_END
    
    
#pxm-QD_CPPCLASS
class DrawingArea (QQuickWidget):
    #pxm-FUNC HD nogil
    def __init__(): pass
        #pxh> : QQuickWidget() {}
    #pxm-FUNC PHD with gil
    def init(this):
        this.setAcceptDrops(True)
        this.setFocusPolicy(Qt.StrongFocus)
        if debug & DEBUG_NOCONTROLS:
            this.setMinimumSize(128, 128)  # for thumbnail generation
        else:
            this.setMinimumSize(300, 300)
        if debug & DEBUG_DRAW:
            this.setMouseTracking(True)
#pxm>CPPCLASS_END


#pxm-QD_CPPCLASS
class PreferencesDialog (QDialog):
    #pxh>QStandardItemModel *liststore_movekeys;
    #pxd+QStandardItemModel *liststore_movekeys
    #pxh>bool liststore_blocked;
    #pxd+cbool liststore_blocked
    #pxh>QString current_facekey;
    #pxd+QString current_facekey
    #pxh>QString image_dirname;
    #pxd+QString image_dirname
    
    #pxm-FUNC HD nogil
    def __init__(parent:'QWidget *'): pass
        #pxh> : QDialog(parent), liststore_blocked(false) {
        #pxh>    this->liststore_movekeys = new QStandardItemModel(this);
        #pxh>}
#pxh>
    #pxm-FUNC PH with gil
    def slot_on_slider_animspeed_valueChanged(this, value:int):
        settings['draw.speed'] = value
    #pxm-FUNC PH with gil
    def slot_on_button_animspeed_reset_clicked(this):
        this.findChild(str2q('slider_animspeed')).setProperty(b'value', tovariant(settings['draw.speed_default']))
        del settings['draw.speed']
    #pxm-FUNC PH with gil
    def slot_on_combobox_shader_currentIndexChanged(this, value:int):
        settings['draw.shader'] = value
    #pxm-FUNC PH with gil
    def slot_on_button_shader_reset_clicked(this):
        this.findChild(str2q('combobox_shader')).setProperty(b'currentIndex', tovariant(settings['draw.shader_default']))
        del settings['draw.shader']
    #pxm-FUNC PH with gil
    def slot_on_combobox_samples_currentIndexChanged(this, value:int):
        settings['draw.samples'] = value
        #visible = (this.sample_buffers != 2**settings['draw.samples'])
        this.findChild(str2q('label_needs_restarted')).setProperty(b'visible', tovariant(True))
    #pxm-FUNC PH with gil
    def slot_on_button_antialiasing_reset_clicked(this):
        this.findChild(str2q('combobox_samples')).setProperty(b'currentIndex', tovariant(settings['draw.samples_default']))
        del settings['draw.samples']
    #pxm-FUNC PH with gil
    def slot_on_checkbox_mirror_faces_toggled(this, checked:bool):
        uidata.preferences.findChild(str2q('spinbox_mirror_faces')).setProperty(b'enabled', tovariant(checked))
        settings['draw.mirror_faces'] = checked
    #pxm-FUNC PH with gil
    def slot_on_spinbox_mirror_faces_valueChanged(this, value:'double'):
        settings['draw.mirror_distance'] = value
    #pxm-FUNC PH with gil
    def slot_on_button_mirror_faces_reset_clicked(this):
        this.findChild(str2q('checkbox_mirror_faces')).setProperty(b'checked', tovariant(settings['draw.mirror_faces_default']))
        del settings['draw.mirror_faces']
        this.findChild(str2q('spinbox_mirror_faces')).setProperty(b'value', tovariant(settings['draw.mirror_distance_default']))
        del settings['draw.mirror_distance']
            
    #pxm-FUNC PHD
    def fill_move_key_list(this):
        this.liststore_blocked = True
        #px+cdef QStandardItem *item
        for i, (move, key) in enumerate(settings['draw.accels']):
            #px+item = new QStandardItem(str2q(move))
            this.liststore_movekeys.setItem(i, 0, item)
            #px+item = new QStandardItem(str2q(key))
            this.liststore_movekeys.setItem(i, 1, item)
        #px+item = new QStandardItem(str2q(_('Move')))
        this.liststore_movekeys.setHorizontalHeaderItem(0, item)
        #px+item = new QStandardItem(str2q(_('Key')))
        this.liststore_movekeys.setHorizontalHeaderItem(1, item)
        this.liststore_blocked = False
        
    #pxm-FUNC PHD
    def get_move_key_list(this)->object:
        move_keys = []
        row_count = this.liststore_movekeys.rowCount()
        for i in range(row_count):
            move = fromvariant(this.liststore_movekeys.item(i, 0).data(Qt.DisplayRole))
            key = fromvariant(this.liststore_movekeys.item(i, 1).data(Qt.DisplayRole))
            move_keys.append((move, key))
        return move_keys
        
    #pxm-FUNC PH with gil
    def slot_on_liststore_movekeys_itemChanged(this, unused_item:'QStandardItem *'):
        if this.liststore_blocked:  return
        settings['draw.accels'] = this.get_move_key_list()
        
    #pxm-FUNC PH with gil
    def slot_on_button_movekey_add_clicked(this):
        this.liststore_blocked = True
        qtui.add_movekey_row()
        this.liststore_blocked = False
        
    #pxm-FUNC PH with gil
    def slot_on_button_movekey_remove_clicked(this):
        qtui.remove_movekey_row()
        settings['draw.accels'] = this.get_move_key_list()
        
    #pxm-FUNC PH with gil
    def slot_on_button_movekey_reset_clicked(this):
        del settings['draw.accels']
        this.fill_move_key_list()
        
    #pxm-FUNC PH with gil
    def slot__on_listview_faces_currentRowChanged(this, current:'const QModelIndex &'):
        this.current_facekey = qtui.get_liststore_faces_facekey(current)
        pydata.app.on_dialog_change_current_face(q2str(this.current_facekey))
        
    #pxm-FUNC PH with gil
    def slot_on_button_color_clicked(this):
        #px+cdef QString color
        current_facekey = q2str(this.current_facekey)
        color = str2q(settings['theme.faces',current_facekey,'color'])
        color = qtui.color_dialog(this, color)
        if q2str(color):
            qtui.set_button_color(color)
            settings['theme.faces',current_facekey,'color'] = q2str(color)
            
    #pxm-FUNC PH with gil
    def slot_on_button_color_reset_clicked(this):
        current_facekey = q2str(this.current_facekey)
        del settings['theme.faces',current_facekey,'color']
        qtui.set_button_color(str2q(settings['theme.faces',current_facekey,'color']))
        
    #pxm-FUNC PH with gil
    def slot_on_combobox_image_activated(this, index:int):
        pydata.app.on_dialog_change_current_image(q2str(this.current_facekey), index)
        
    #pxm-FUNC PH with gil
    def slot_on_button_image_reset_clicked(this):
        pydata.app.on_dialog_reset_current_image(q2str(this.current_facekey))
        
    #pxm-FUNC PH with gil
    def slot_on_radiobutton_tiled_toggled(this, checked:bool):
        if checked:
            settings['theme.faces',q2str(this.current_facekey),'mode_nick'] = 'tiled'
    #pxm-FUNC PH with gil
    def slot_on_radiobutton_mosaic_toggled(this, checked:bool):
        if checked:
            settings['theme.faces',q2str(this.current_facekey),'mode_nick'] = 'mosaic'
            
    #pxm-FUNC PH with gil
    def slot_on_button_background_color_clicked(this):
        #px+cdef QString color
        color = str2q(settings['theme.bgcolor'])
        color = qtui.color_dialog(this, color)
        if q2str(color):
            qtui.set_button_background_color(color)
            settings['theme.bgcolor'] = q2str(color)
            
    #pxm-FUNC PH with gil
    def slot_on_button_background_color_reset_clicked(this):
        del settings['theme.bgcolor']
        qtui.set_button_background_color(str2q(settings['theme.bgcolor']))
#pxm>CPPCLASS_END

def set_combobox_current_image(index_icon, imagefile, index):
    #px+cdef QIcon icon
    if imagefile is not None:
        icon.addFile(str2q(imagefile))
    qtui.set_combobox_image(index_icon, icon, index)
    
def set_preferences_current_face_theme(color, imageindex_icon, imagefile, imageindex, imagemode):
    qtui.set_button_color(str2q(color))
    set_combobox_current_image(imageindex_icon, imagefile, imageindex)
    qtui.set_imagemode(imagemode)
    
def get_filedialog_imagefile():
    filename = q2str(getOpenFileName(uidata.preferences, str2q(_("Open Image")), uidata.preferences.image_dirname))
    if filename:
        uidata.preferences.image_dirname = str2q(os.path.dirname(filename))
    return filename
#pxm>IF_END
    
def preferences_dialog(shader_names, sample_buffers, facenames, stockicons):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+uidata.preferences = new PreferencesDialog(uidata.mainwindow)
    speed_min, speed_max = settings['draw.speed_range']
    mirror_min, mirror_max = settings['draw.mirror_distance_range']
    qtui.setupUi_pref(uidata.preferences,
                    settings['draw.speed'], speed_min, speed_max,
                    mirror_min, mirror_max, uidata.preferences.liststore_movekeys)
    qtui.preferences_block_signals(True)
    
    # graphic tab
    for nick in settings['draw.shader_range']:
        qtui.combobox_add_shaderitem(str2q(shader_names.get(nick, nick)), str2q(nick))
    for text in settings['draw.samples_range']:
        qtui.combobox_add_samplesitem(str2q(_(text)), str2q(text))
    qtui.set_prefs_graphics_values(settings['draw.shader'], settings['draw.samples'],
                                   settings['draw.mirror_faces'], settings['draw.mirror_distance'])
    
    # keys tab
    uidata.preferences.fill_move_key_list()
    
    # theme tab
    for facename, facekey in facenames:
        qtui.add_liststore_faces_row(str2q(facename), str2q(facekey))
    qtui.add_combobox_image_item(QIcon(), str2q(_('plain')), str2q(''))
    for filename, image_func in stockicons:
        image = image_func(filename)
        qtui.add_combobox_image_item(image_to_icon(image), str2q(''), str2q(filename))
    qtui.add_combobox_image_item(QIcon(), str2q(_('select …')), str2q('/'))
    uidata.preferences.image_dirname = str2q(get_pictures_folder())
    qtui.finalize_liststore_faces(uidata.preferences)
    
    qtui.set_button_background_color(str2q(settings['theme.bgcolor']))
    
    qtui.preferences_block_signals(False)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    ctx_set_stdmodel(b'shadermodel', ([shader_names.get(nick, nick), nick] for nick in settings['draw.shader_range']))
    ctx_set(b'activesamples', sample_buffers)
    ctx_set(b'sampleindex', len(bin(sample_buffers)[3:]))
    ctx_set_stdmodel(b'samplemodel', ([_(text), text] for text in settings['draw.samples_range']))
    
    ctx_set_stdmodel(b'movekey_model', settings['draw.accels'])
    
    ctx_set_stdmodel(b'facesmodel', facenames)
    images = [[fn and os.path.join(config_.UI_DIR, 'images', fn), fn] for fn, unused in stockicons]
    ctx_set_stdmodel(b'imagemodel', images)
    #pxm>IF_END
    
def show_preferences():
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    uidata.preferences.show()
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    ctx_set(b'preferences_visible', True)
    #pxm>IF_END
    
def initial_resize():
    #pxm>IF '[[ANDROID]]' != 'android'
    width, height = settings['window.size']
    uidata.mainwindow.resize(width, height)
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+cdef int divider_pos = settings['window.divider']
    uidata.mainwindow.show()
    qtui.set_splitter_sizes(divider_pos)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    uidata.mainwindow.show()
    #pxm>IF_END
    singleShot(20, _on_init_finalize)
    
def set_title(title):
    uidata.mainwindow.setTitle(str2q(title))
    
def load_ui(path):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #uidata.engine.clearComponentCache()
    if path is None:
        path = os.path.join(config_.UI_DIR, 'qt', 'DrawingArea.qml')
    uidata.drawingarea.setResizeMode(SizeRootObjectToView)
    uidata.drawingarea.setSource(fromLocalFile(str2q(path)))
    while uidata.drawingarea.status() == QQuickWidget_Loading:
        processEvents()
    #px+cdef QmlErrorList errors
    errors = uidata.drawingarea.errors()
    if errors.size() > 0:
        print_warnings(errors)
    
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    if path is None:
        path = os.path.join(config_.UI_DIR, 'qt', 'main.qml')
    if uidata.component:
        uidata.component.disconnect()
        del uidata.component
        uidata.component = NULL
    if uidata.mainwindow:
        uidata.mainwindow.disconnect()
        del uidata.mainwindow
        uidata.mainwindow = NULL
        
    uidata.engine.clearComponentCache()
    #px+uidata.component = new QQmlComponent(uidata.engine, NULL)
    uidata.component.loadUrl(fromLocalFile(str2q(path)))
    while uidata.component.isLoading():
        processEvents()
    #px+cdef QmlErrorList errors
    errors = uidata.component.errors()
    if errors.size() > 0:
        print_warnings(errors)
    assert uidata.component.isReady()
    
    #px+cdef QObject *rootobject
    rootobject = uidata.component.create()
    assert rootobject and rootobject.inherits('QQuickWindow')
    
    #px+uidata.mainwindow = <QQuickWindow*>rootobject
    uidata.engine.setIncubationController(uidata.mainwindow.incubationController())
    #pxm>IF_END
        
def close_mainwindow():
    uidata.mainwindow.close()
    
def get_samples():
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #TODO: uidata.drawingarea.format().samples() returns only 0 here
    return defaultFormat().samples()
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    return uidata.mainwindow.format().samples()
    #pxm>IF_END
    
def save_screenshot(filename):
    pydata.screenshot_filename = filename
    ctx_invoke(b'capture_filename_changed')
    
def grab_screenshot(filename):
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+cdef QPixmap image
    image = uidata.mainwindow.grab()
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    #px+cdef QImage image
    image = uidata.mainwindow.grabWindow()
    #pxm>IF_END
    if not (debug & DEBUG_SIM):
        image.save(str2q(filename), NULL, -1)
        
        
#### qt replacement for functions in os module to support 'qrc:' and 'assets:' ####
#XXX: maybe use the python open function on other platforms,
def fs_list_filenames_sorted(path):
    #px+cdef QDir qdir = QDir(str2q(path))
    #px+cdef QFileInfoList qlist = qdir.entryInfoList(QDir_Files)
    for i in range(qlist.size()):
        yield q2str(qlist.at(i).fileName())
def fs_list_dir(dirname):
    #px+cdef QDir qdir
    qdir = QDir(str2q(dirname))
    if not qdir.exists():
        yield None, None
        return
    #px+cdef QFileInfoList qlist = qdir.entryInfoList(QDir_Files_Readable, QDir_Name_IgnoreCase)
    #px+cdef QFileInfo qfi
    for i in range(qlist.size()):
        qfi = qlist.at(i)
        yield q2str(qfi.filePath()), q2str(qfi.suffix())
def os_path_exists(path):
    return QFileInfo_exists(str2q(path))
def fs_read_file(filename):
    #px+cdef QFile file
    #px+cdef QByteArray data
    file.setFileName(str2q(filename))
    if not file.open(ReadOnly):
        print('cannot open', filename)
        return None
    data = file.readAll()
    return data.data()[:data.size()]
    
    
#### misc qt ####

def parse_color(name):
    #px+cdef QColor color
    color.setNamedColor(str2q(name))
    return color.red(), color.green(), color.blue()
    
def parse_color_f(name):
    #px+cdef QColor color
    color.setNamedColor(str2q(name))
    return color.redF(), color.greenF(), color.blueF()
    
#px/cdef class Image:
class Image:
    #px+cdef QImage image
    def width(self):  return self.image.width()
    def height(self):  return self.image.height()
    def data(self):  return self.image.bits()[:self.image.byteCount()]  #TODO: byteCount obsolete in qt 5.10 use sizeInBytes
    #def data(self):  return self.image.bits()[:self.image.sizeInBytes()]
    def load(self, filename, maxsize):
        #px+cdef int width, height, scaled_width, scaled_height
        #px+cdef bytes data
        self.image.load(str2q(filename), NULL)
        if self.image.isNull():
            return None
        # scale the image to size 2^n for OpenGL
        width = self.image.width()
        height = self.image.height()
        if width < 1 or height < 1:
            return None
        scaled_width = maxsize
        while scaled_width > width:
            scaled_width //= 2
        scaled_height = maxsize
        while scaled_height > height:
            scaled_height //= 2
        self.image = self.image.scaled(scaled_width, scaled_height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
        self.image = self.image.convertToFormat(Format_RGBA8888, Qt.AutoColor)
        return self
    #FIXME: for cppclass remove first arg, for cdef class not
    #pxm-FUNC P
    def set_empty(this, self)->'Image':
        self.image = QImage(1,1,Format_RGBA8888)
        self.image = self.image.convertToFormat(Format_RGBA8888, Qt.AutoColor)
        self.image.fill(0)
        return self
#px+cdef Image _empty_image = Image().set_empty()
def empty_image():
    return _empty_image
    
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxm-FUNC P
def image_to_icon(image:'Image')->'QIcon':
    #px+cdef QPixmap pixmap
    if image is None:
        return QIcon()
    else:
        pixmap = QPixmap(image.image.width(), image.image.height())
        pixmap.convertFromImage(image.image)
        return QIcon(pixmap)
#pxm>IF_END
        
def keyval_from_name(keystr):
    keystr = '+'.join(('Num' if k == 'KP' else k) for k in keystr.split('+'))
    #px+cdef QKeySequence keyseq = fromString(str2q(keystr), PortableText)
    if keyseq.count() == 0:
        return 0
    return keyseq[0]
    
def get_pictures_folder():
    #px+cdef QStringList locations = standardLocations(PicturesLocation)
    if locations.size() <= 0:
        locations = standardLocations(HomeLocation)
        if locations.size() <= 0:
            return ''
    return q2str(locations[0])
    
def get_imagefile(facekey):
    imagefile = settings['theme.faces',facekey,'image']
    currentfolder = get_pictures_folder()
    if imagefile.startswith('/'):
        currentfolder = os.path.dirname(imagefile)
    elif imagefile:
        imagefile = os.path.join(config_.UI_DIR, 'images', imagefile)
    return currentfolder, imagefile
    
def text_to_html(text):
    return q2str(fromPlainText(str2q(text)).toHtml())
    
#pxm-FUNC P with gil
def _on_filesystemwatcher_changed_1(qpath:str):
    if pydata.filesystemwatcher_path is not None:
        return
    pydata.filesystemwatcher_path = q2str(qpath)
    singleShot(100, _on_filesystemwatcher_changed_2)
#pxm-FUNC P with gil
def _on_filesystemwatcher_changed_2():
    path = pydata.filesystemwatcher_path
    pydata.filesystemwatcher_path = None
    pydata.filesystemwatcher_handlers[path]()
    if os.path.isfile(path):
        uidata.filesystemwatcher.addPath(str2q(path))
    
def create_filesystemwatcher(path, callback):
    #px+cdef QString qpath = str2q(path)
    if uidata.filesystemwatcher is NULL:
        #px+uidata.filesystemwatcher = new QFileSystemWatcher(NULL)
        #px+connect(uidata.filesystemwatcher, &directoryChanged, _on_filesystemwatcher_changed_1)
        #px+connect(uidata.filesystemwatcher, &fileChanged, _on_filesystemwatcher_changed_1)
        pass
    pydata.filesystemwatcher_handlers[path] = callback
    uidata.filesystemwatcher.addPath(qpath)
    
#XXX: avoid this function, only one active singleShot-handler
#pxm-FUNC P with gil
def _on_singleShot():
    pydata.singleshot_handler()
    if pydata.singleshot_queue:
        msec, pydata.singleshot_handler = pydata.singleshot_queue.pop(0)
        singleShot(msec, _on_singleShot)
    else:
        pydata.singleshot_handler = None
def timer_singleShot(msec, func):
    if pydata.singleshot_handler is None:
        pydata.singleshot_handler = func
        singleShot(msec, _on_singleShot)
    else:
        pydata.singleshot_queue.append((msec, func))
    
#pxm-FUNC P with gil
def _on_settings_timer_timeout():
    pydata.app.on_settings_timer_timeout()
    
#pxm-FUNC P
def settings_timer_create():
    #px+uidata.settings_timer = new QTimer()
    uidata.settings_timer.setInterval(5000)
    uidata.settings_timer.setSingleShot(True)
    #px+connect(uidata.settings_timer, &timeout, _on_settings_timer_timeout)
    
#pxm-FUNC P
def settings_timer_start()->'cpdef':
    if not uidata.settings_timer.isActive():
        uidata.settings_timer.start()
        
#pxm-FUNC P
def settings_timer_destroy():
    uidata.settings_timer.stop()
    uidata.settings_timer.deleteLater()
    uidata.settings_timer = NULL
    
def set_animation_speed():
    ctx_invoke(b'animation_speed_changed')
    
#pxm-FUNC P
def animation_init():
    ctx_set(b'animation_running', False)
    set_animation_speed()
    
#pxm-FUNC P
def animate_timer_start(msec, angle_max)->'cpdef':
    ctx_set(b'animation_maxangle', angle_max)
    ctx_set(b'animation_running', True)
    
def animate_timer_stop():
    ctx_set(b'animation_running', False)
    
#pxm-FUNC P
def destroy_resources():
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    uidata.drawingarea.setMouseTracking(False)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    settings['window.size'] = uidata.mainwindow.width(), uidata.mainwindow.height()
    #pxm>IF_END
    
#pxm>IF '[[QTVARIANT]]' == 'qtw'
#pxm-FUNC P
def setupUi_main(window:'QMainWindow *', edit:'QLineEdit *', drawing:'QWidget *', status:'QLabel *'):
    from pybiklib import config as config_
    qtui.setupUi_main(window, 
                     str2q(config_.APPICON_FILE),
                     str2q(os.path.join(config_.ICONS_DIR, 'new-random.png')),
                     str2q(os.path.join(config_.ICONS_DIR, 'new-solved.png')),
                     str2q(os.path.join(config_.ICONS_DIR, 'select-model.png')),
                     str2q(settings['action.edit_moves']),
                     str2q(settings['action.edit_cube']))
    qtui.set_visible(settings['window.editbar'], settings['window.statusbar'])
    qtui.set_shortcuts(str2q(settings['action.selectmodel']),
                       str2q(settings['action.initial_state']),
                       str2q(settings['action.reset_rotation']),
                       str2q(settings['action.preferences']),
                      )
    qtui.add_widgets(window, edit, drawing, status)
    if debug & DEBUG_NOCONTROLS:
        qtui.hide_controls()
    
#pxm>IF '[[QTVARIANT]]' == 'qtq'
#pxm-FUNC CHD nogil
def connect_engine(engine:'QQmlEngine *'):  pass
    #pxc>{ QObject::connect((QObject*)engine, SIGNAL(quit()), QCoreApplication::instance(), SLOT(quit())); }
#pxm>IF_END

def create_window(pyapp):
    pydata.app = pyapp
    init_module()
    # Set default format before any widget is created, so that everything uses the same format.
    # To test this use DEBUG_VFPS. The framerate should be then >60.
    set_default_surface_format()
    
    #pxm>IF '[[QTVARIANT]]' == 'qtw'
    #px+uidata.mainwindow = new MainView()
    #px+uidata.treestore = new QStandardItemModel()
    #px+uidata.drawingarea = new DrawingArea()
    uidata.drawingarea.init()
    #px+uidata.move_edit = new MoveEdit()
    #px+uidata.statuslabel = new QLabel()
    uidata.engine = uidata.drawingarea.engine()
    setupUi_main(uidata.mainwindow, uidata.move_edit, uidata.drawingarea, uidata.statuslabel)
    
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    #px+uidata.engine = new QQmlEngine()
    #pxm>IF_END
    uidata.engine.setOutputWarningsToStandardError(False)
    #px+connect(uidata.engine, &warnings, print_warnings)
    #px+uidata.ctx = <QObject*>(new ContextProperty())
    uidata.engine.rootContext().setContextProperty(str2q('ctx'), uidata.ctx)
    if debug & DEBUG_QML:
        dump_info('Context Property', uidata.ctx)
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    connect_engine(uidata.engine)
    #pxm>IF_END
    
    animation_init()
    settings_timer_create()
    
def destroy_window():
    settings_timer_destroy()
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    del uidata.component
    uidata.component = NULL
    #pxm>IF_END
    pydata.app = None
    
#pxm-FUNC P with gil
def _on_init_finalize():
    pydata.app.on_init_finalize()
    
#pxm>IF '[[QTVARIANT]]' == 'qtq'
#pxm-FUNC P with gil
def _on_closing():
    destroy_resources()
    pydata.app.on_closing()
    pydata.app = None
#pxm>IF_END
        
def app_post_create(gettext):
    #pxm>IF '[[FROZENQTQ]]' != 'frozenqtq'
    #px+cdef QApplication *app = <QApplication*>QCoreApplication_instance()
    #pxm>IF '[[FROZENQTQ]]' == 'frozenqtq'
    #px+cdef QGuiApplication *app = <QGuiApplication*>QCoreApplication_instance()
    #pxm>IF_END
    
    global _
    _ = gettext
    
    #pxm>IF '[[QTVARIANT]]' == 'qtq'
    # variant 'qtw' uses closeEvent instead
    #px+connect(app, &aboutToQuit, app, &_on_closing, Qt.DirectConnection)
    #pxm>IF_END
    init_qt_impl(vertexbuffer_stride)
    
    

