#!/usr/bin/python3
# -*- coding: utf-8 -*-

#  Copyright © 2018  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from contextlib import suppress

from pybiklib.settings import settings
from pybiklib import pluginlib
from pybiklib.model import Model
from pybiklib.utils import exc_end
from pybiklib.ext import qt

def irun(app):
    yield 100
    yield 100
    
    def filter_solutions():
        for mtype in Model.cache_index['types']:
            for size in sorted(set(Model.cache_index['normsize'][mtype].values())):
                # use noload: need only the type and sizes attributes, saves a lot of time
                model = Model(mtype, size, noload=True)
                for funcidx, (path, script) in enumerate(app.plugin_helper.scripts.items()):
                    # only test solutions
                    if len(path) != 2 or path[0][0] != 'Solvers':
                        continue
                    # only test compatible solutions, other solutions would display a message box
                    try:
                        app.plugin_helper.get_function(model, funcidx)
                    except pluginlib.PluginModelCompatError:
                        continue
                    yield mtype, size, funcidx, '/'.join(p for p,t in path), script
    print('filter solutions...')
    solutions = list(filter_solutions())
    yield 100
    
    for mtype, size, func_idx, path, scripts in solutions:
        app.ui_load_other_game(mtype, size)
        print(f'{mtype} {size} #{func_idx}:', path)
        yield 100
        
        for i in range(20):
            print(' ', i+1)
            app.ui_new_game(solved=False)
            yield 100
            app.ui_plugin_activated(func_idx)
            yield 100
            app.on_action_forward_triggered()
            yield 100
            assert app.game.current_state.is_solved()
    qt.close_mainwindow()
    
def run(app):
    it = irun(app)
    @exc_end
    def step():
        with suppress(StopIteration):
            delay = next(it)
            qt.timer_singleShot(delay, step)
    step()
    

