#!/usr/bin/python3
# -*- coding: utf-8 -*-

#  Copyright © 2017  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function, division

import sys
sys.path.insert(0, '.')
from contextlib import suppress

from pybiklib.model import Model


def check_parallel(iaxis1, slice1, iaxis2, slice2, model):
    from pybiklib.cubestate import CubeState
    state = CubeState(model)
    state.set_solved()
    state.rotate_slice((iaxis1, slice1, False))
    state.rotate_slice((iaxis2, slice2, False))
    state.rotate_slice((iaxis1, slice1, True))
    state.rotate_slice((iaxis2, slice2, True))
    return state.is_identity()
    
def main():
    Model.load_index()
    for mtype in Model.cache_index['types']:
        for size in sorted(set(Model.cache_index['normsize'][mtype].values())):
            print(mtype, size, file=sys.stderr)
            model = Model(mtype, size)
            for iaxis1, (unused_axis1, dim1) in enumerate(zip(model.axes, model.sizes)):
                for iaxis2, (unused_axis2, dim2) in enumerate(zip(model.axes, model.sizes)):
                    if iaxis2 <= iaxis1:
                        continue
                    for slice1 in range(dim1):
                        for slice2 in range(dim2):
                            res1 = check_parallel(iaxis1, slice1, iaxis2, slice2, model)
                            res2 = model.is_parallel(iaxis1, slice1, iaxis2, slice2)
                            if res1:
                                print(mtype, size, ':', iaxis1, slice1, '-', iaxis2, slice2, flush=True)
                            elif res2:
                                print('X', mtype, size, ':', iaxis1, slice1, '-', iaxis2, slice2, flush=True, file=sys.stderr)
                            assert res1 == res2, (res1, res2)
                            
    
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass
        

