# -*- coding: utf-8 -*-

#  Copyright © 2017, 2019  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from contextlib import suppress
from pathlib import Path
from subprocess import check_call

from pybiklib import config
from pybiklib.settings import settings
from pybiklib.model import Model
from pybiklib.utils import exc_end
from pybiklib.ext import qt

def set_angle(app, angle):
    move_data = app.game.move_sequence.current()
    blocks = app.game.current_state.identify_rotation_blocks(move_data.axis, move_data.slice)
    angle_max = 360. / app.model.symmetries[move_data.axis]
    axisx, axisy, axisz = app.model.axes[move_data.axis]
    if move_data.dir:
        qt.set_animation_start(blocks, -axisx, -axisy, -axisz)
    else:
        qt.set_animation_start(blocks, axisx, axisy, axisz)
    yield 100
    qt.set_animation_next(angle)
    yield 100
    
def irun(app):
    wsize = 1024
    settings['window.size'] = (wsize, wsize)
    settings['draw.samples'] = 3
    settings['draw.zoom'] = 1.44
    settings['draw.speed'] = 10
    dirname = Path(config.appdata_dir, 'icons')
    dirname.mkdir(exist_ok=True)
    
    yield 1000  # Wait until the app loaded the current game
    print()
    mtype, size = app.ui_load_game('Cube', (3,))
    print('Model:', mtype, size)
    yield 100
    app.camrotation_changed(6, -20)
    yield 100
    if True:
        app.on_edit_finished('u', 1)
    else:
        app.on_edit_finished('u', 0)
        yield 100
        yield from set_angle(app, 80)
    yield 100
    
    filename_tmp = dirname / 'pybik-tmp.png'
    print('  screenshot:', str(filename_tmp.relative_to(Path().absolute())))
    qt.save_screenshot(str(filename_tmp))
    yield 100
    qt.close_mainwindow()
    
    psize = 1024
    dx = -25
    dy = 40
    offs = max(abs(dx), abs(dy))
    lsize = psize + 2*offs
    print('  size %s' % psize)
    filename = dirname / ('pybik-%s.png' % psize)
    check_call(['convert', str(filename_tmp),
                    '-background','#00000000','-compose','Copy','-gravity','center',
                    '-extent','{0}x{0}'.format(lsize),
                    '-crop','{0}x{0}+{1}+{2}'.format(psize,dx,dy),
                    str(filename)])
    filename_tmp.unlink()
    
    # Launchpad: 14x14, 64x64, 192x192
    for psize in 14, 22, 24, 32, 48, 64, 128, 192, 256, 572:
        print('  size %s' % psize)
        pfilename = dirname / ('pybik-%s.png' % psize)
        check_call(['convert', str(filename), '-resize', '{0}x{0}'.format(psize), str(pfilename)])
    alpha = str(dirname / 'pybik-572.png')
    white = str(dirname / 'pybik_whitebg-572.png')
    check_call(['convert', alpha, '-background', 'white', '-alpha', 'remove', white])
    yield 0
    
def run(app):
    it = irun(app)
    @exc_end
    def on_step():
        with suppress(StopIteration):
            delay = next(it)
            #print('timer_singleShot', delay, on_step)
            qt.timer_singleShot(delay, on_step)
    on_step()
    

