# -*- coding: utf-8 -*-

#  Copyright © 2017-2018  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from contextlib import suppress
from subprocess import check_call
from pathlib import Path
import shutil

from pybiklib import config
from pybiklib.settings import settings
from pybiklib.model import Model
from pybiklib.utils import exc_end
from pybiklib.cubestate import CubeState
from pybiklib.ext import qt

def irun(app):
    wsize = (256, 256)
    settings['window.size'] = wsize
    yield 100
    settings['window.size'] = wsize
    yield 100
    dirname = os.path.join(config.UI_DIR, 'qt', 'images')
    with suppress(FileExistsError):
        os.mkdir(dirname)
    with suppress(FileExistsError):
        os.mkdir(dirname)
    print()
    
    mtype, size = app.ui_load_game('Cube', (3,))
    print('Model:', mtype, size)
    yield 100
    app.camrotation_changed(4, -12)
    settings['draw.zoom'] = 0.7
    
    yield 100
    print('  solved')
    filename_solved = os.path.join(dirname, 'new-solved-tmp.png')
    qt.save_screenshot(filename_solved)
    print('    screenshot:', filename_solved)
    
    yield 0
    print('  random')
    CubeState.rand.seed(134)
    app.ui_new_game(solved=False)
    yield 100
    filename_random = os.path.join(dirname, 'new-random-tmp.png')
    qt.save_screenshot(filename_random)
    print('    screenshot:', filename_random)
    
    yield 100
    qt.close_mainwindow()
    
    def hires_name(path, tag):
        return str(path.with_name(path.stem + tag + path.suffix))
    def iconpath(size, name):
        p = f'/usr/share/icons/HighContrast/{size}x{size}/actions/{name}'
        assert os.path.exists(p), p
        return p
        
    for fnt in filename_solved, filename_random:
        fnt2 = fnt.replace('-tmp','-tmp2')
        fn = Path(fnt.replace('-tmp',''))
        check_call(['convert', fnt, '-crop', '128x128+62+68', fnt2])
        os.remove(fnt)
        check_call(['convert', str(fnt2), '-resize', '22x22', str(fn)])
        check_call(['convert', str(fnt2), '-resize', '48x48', hires_name(fn, '@2x')])
        check_call(['convert', str(fnt2), '-resize', '72x72', hires_name(fn, '@3x')])
        check_call(['convert', str(fnt2), '-resize', '96x96', hires_name(fn, '@4x')])
        os.remove(fnt2)
    custom_filenames = [Path(f).stem.replace('-tmp','') for f in (filename_solved, filename_random)] + ['select-model']
    for path in Path(config.UI_DIR, 'qt', 'images').iterdir():
        if path.suffix == '.png' and '@' not in path.stem and path.stem not in custom_filenames:
            shutil.copy(iconpath(22, path.name), path)
            shutil.copy(iconpath(48, path.name), hires_name(path, '@2x'))
            src = iconpath(256, path.name)
            check_call(['convert', src, '-resize', '72x72', hires_name(path, '@3x')])
            check_call(['convert', src, '-resize', '96x96', hires_name(path, '@4x')])
    
def run(app):
    it = irun(app)
    @exc_end
    def on_step():
        with suppress(StopIteration):
            delay = next(it)
            #print('timer_singleShot', delay, on_step)
            qt.timer_singleShot(delay, on_step)
    on_step()
    

