# -*- coding: utf-8 -*-

#  Copyright © 2017, 2019  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from contextlib import suppress
from pathlib import Path
from subprocess import check_call, Popen

from pybiklib import config
from pybiklib.settings import settings
from pybiklib.moves import MoveFormat
from pybiklib.utils import exc_end
from pybiklib.ext import qt

def set_angle(app, steps, code=None):
    if code is None:
        move_data = app.game.step_next()
    else:
        move_data = MoveFormat.parse(code, app.model)[0]
    blocks = app.game.current_state.identify_rotation_blocks(move_data.axis, move_data.slice)
    angle_max = 360. / app.model.symmetries[move_data.axis]
    axisx, axisy, axisz = app.model.axes[move_data.axis]
    if move_data.dir:
        qt.set_animation_start(blocks, -axisx, -axisy, -axisz)
    else:
        qt.set_animation_start(blocks, axisx, axisy, axisz)
    yield 100
    qt.set_animation_next(10)
    yield 100
    
def take_screenshot(filename):
    print('  screenshot:', str(filename.relative_to(Path().absolute())))
    #qt.save_screenshot(str(filename))
    p = Popen(('shutter', '--active', '-o', str(filename), '--no_session', '--exit_after_capture'))
    yield 5000
    p.wait()
    print('  done.')
    yield 100
    
def sh1(app):
    mtype, size = app.ui_load_game('Cube', (3,))
    print('Model:', mtype, size)
    yield 100
    app.on_edit_finished('b-r-fu-l', 8)
    app.game.set_as_initial_state()
    app.ui_update()
    yield 100
    app.on_edit_finished('l-u', 3)
    yield 100
    check_call(['xte'] +
                       # sidepane->Challenges->Solve in 5 moves
                       ['key Tab', 'key Space', 'key Tab'] +
                       ['key Down'] * 5)
    yield 100
    yield from set_angle(app, 5, 'f-')
    yield 100
    
def sh2(app):
    settings['theme.bgcolor'] = '#b65b4b'
    settings['theme.faces.back.color'] = '#936992'
    settings['theme.faces.back.image'] = 'diagonal-lines.png'
    settings['theme.faces.back.mode_nick'] = 'mosaic'
    settings['theme.faces.down.color'] = '#961144'
    settings['theme.faces.front.color'] = '#785696'
    settings['theme.faces.front.image'] = 'SHAMROCK.png'
    settings['theme.faces.front.mode_nick'] = 'mosaic'
    settings['theme.faces.left.color'] = '#e36a6c'
    settings['theme.faces.left.image'] = 'SHAMROCK.png'
    settings['theme.faces.right.color'] = '#9d415d'
    settings['theme.faces.right.image'] = 'PEACE SYMBOL.png'
    settings['theme.faces.right.mode_nick'] = 'tiled'
    settings['theme.faces.up.color'] = '#cf4dba'
    settings['theme.faces.up.image'] = 'WHITE SMILING FACE.png'
    settings['theme.faces.up.mode_nick'] = 'tiled'
    settings['draw.mirror_faces'] = True
    settings['draw.zoom'] = 1.92
    settings['draw.mirror_distance'] = 1.8
    yield 100
    mtype, size = app.ui_load_game('Cube', (4,))
    print('Model:', mtype, size)
    yield 100
    check_call(['xte'] +
                       # ->Pretty patterns->Cube in a Cube
                       ['key Tab'] + ['key Space', 'key Tab'] +
                       ['key Down'] * 12 + ['key Up'] * 2 + ['key Return', ])
    yield 100
    app.game.goto_next_mark()
    app.ui_update()
    yield 1000
    
def sh3(app):
    del settings['theme.bgcolor']
    del settings['theme.faces.back.color']
    del settings['theme.faces.back.image']
    del settings['theme.faces.back.mode_nick']
    del settings['theme.faces.down.color']
    del settings['theme.faces.front.color']
    del settings['theme.faces.front.image']
    del settings['theme.faces.front.mode_nick']
    del settings['theme.faces.left.color']
    del settings['theme.faces.left.image']
    del settings['theme.faces.right.color']
    del settings['theme.faces.right.image']
    del settings['theme.faces.right.mode_nick']
    del settings['theme.faces.up.color']
    del settings['theme.faces.up.image']
    del settings['theme.faces.up.mode_nick']
    del settings['draw.mirror_faces']
    del settings['draw.zoom']
    del settings['draw.mirror_distance']
    yield 100
    mtype, size = app.ui_load_game('Tetrahedron', (3,))
    print('Model:', mtype, size)
    yield 1000
    check_call(['xte'] +
                       # ->Volcano
                       ['key Down', 'key Up', 'key Return'])
    yield 100
    app.game.goto_next_mark()
    app.ui_update()
    yield 1000
    
def sh4(app):
    app.animation_stop()
    app.ui_load_other_game('Dodecahedron', (3,))
    print('Model:', app.model.type, app.model.size)
    yield 1000
    check_call(['xte'] +
                       # ->sidepane->Challenges->Solve random puzzle
                       ['key Tab']*4 + ['key Space', 'key Tab'] +
                       ['key Down', 'key Home', 'key Return'])
    yield 100
    app.ui_update()
    yield 1000
    
#def sh5(app):
#    app.animation_stop()
#    app.ui_load_other_game('Brick', (5,1,5))
#    print('Model:', app.model.type, app.model.size)
#    yield 1000
#    check_call(['xte'] +
#                       # ->Solve random puzzle
#                       ['key Home', 'key Return'])
#    yield 100
#    app.ui_update()
#    yield 1000
    
def sh5(app):
    app.animation_stop()
    app.ui_load_other_game('Cube', (3,))
    print('Model:', app.model.type, app.model.size)
    yield 1000
    check_call(['xte'] +
                       # -> Select Puzzle
                       ['keydown Control_L', 'key M', 'keyup Control_L'])
    yield 100
    app.ui_update()
    yield 1000
    
def irun(app):
    wsize = 704
    assert wsize == wsize//16*16
    wsize = wsize, wsize//16*9
    settings['window.size'] = wsize
    settings['window.divider'] = 470
    settings['draw.samples_nick'] = 'high'
    #settings['draw.zoom'] = 1.38
    settings['draw.speed'] = 10
    dirname = Path(config.appdata_dir, 'screenshots')
    dirname.mkdir(exist_ok=True)
    
    yield 100
    print()
    print('Window size:', wsize)
    
    yield from sh1(app)
    yield from take_screenshot(dirname / 'screenshot1.png')
    yield from sh2(app)
    yield from take_screenshot(dirname / 'screenshot2.png')
    yield from sh3(app)
    yield from take_screenshot(dirname / 'screenshot3.png')
    yield from sh4(app)
    yield from take_screenshot(dirname / 'screenshot4.png')
    yield from sh5(app)
    yield from take_screenshot(dirname / 'screenshot5.png')
    qt.close_mainwindow()
    
def run(app):
    it = irun(app)
    @exc_end
    def on_step():
        with suppress(StopIteration):
            delay = next(it)
            #print('timer_singleShot', delay, on_step)
            qt.timer_singleShot(delay, on_step)
    on_step()
    

