# -*- coding: utf-8 -*-

#  Copyright © 2016-2017  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from contextlib import suppress
from subprocess import Popen

from pybiklib import config
from pybiklib.settings import settings
from pybiklib.model import Model
from pybiklib.utils import exc_end
from pybiklib.ext import qt

def irun(app):
    settings['window.size'] = (128, 128)
    yield 100
    i = 0
    process = None
    with suppress(FileExistsError):
        os.mkdir(os.path.join(config.UI_DIR, 'thumbnails'))
    for mtype in Model.cache_index['types']:
        print(mtype)
        for size in sorted(set(Model.cache_index['normsize'][mtype].values())):
            i += 1
            filename = '{}/thumbnails/{}-{}.png'.format(config.UI_DIR, mtype, 'x'.join(str(i) for i in size))
            print(' ', filename)
            app.ui_load_other_game(mtype, size)
            yield 50  # possible race condition, scene may not be rendered
            qt.save_screenshot(filename)
            if process is not None:
                process.wait()
            process = Popen(('optipng', '-quiet', filename))
            yield 0
    yield 100
    qt.close_mainwindow()
    if process is not None:
        process.wait()
    
def run(app):
    it = irun(app)
    @exc_end
    def on_step():
        with suppress(StopIteration):
            delay = next(it)
            qt.timer_singleShot(delay, on_step)
    on_step()
    

