#!/usr/bin/python3
# -*- coding: utf-8 -*-

#  Copyright © 2019  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from pathlib import Path
import re

replacements = [
        ('build/temp.linux-x86_64-3.6/pybiklib/ext', '<BUILD-EXT>'),
        ('build/temp.linux-x86_64-3.6/data/ui/qt', '<BUILD-UIQT>'),
        ('x86_64-linux-gnu-gcc -pthread -DNDEBUG -g -fwrapv -O2 -Wall -g -fstack-protector-strong'
         ' -Wformat -Werror=format-security -Wdate-time -D_FORTIFY_SOURCE=2 -fPIC', '<GCC>'),
        (str(Path.home()/'.local/opt/Qt/5.12.2/gcc_64'), '<QT-DIR>'),
        ('build/temp.linux-x86_64-3.6/<BUILD-EXT>', '<BBUILD-EXT>'),
        ('x86_64-linux-gnu-g++ -pthread -shared'
         ' -Wl,-O1 -Wl,-Bsymbolic-functions -Wl,-Bsymbolic-functions -Wl,-z,relro -Wl,-Bsymbolic-functions', '<G++SO>'),
        ('<G++SO> -Wl,-z,relro -g -fstack-protector-strong -Wformat -Werror=format-security -Wdate-time -D_FORTIFY_SOURCE=2', '<G++SO>'),
        (str(Path('pybiklib/ext').absolute()), '<TRUNK-EXT>'),
        ('-lQt5Core -lQt5Gui -lQt5Widgets', '<LQTW>'),
        ('-lQt5Qml -lQt5Quick -lQt53DRender', '<LQTQ>'),
        ('x86_64-linux-gnu-g++ -pthread -DNDEBUG -g -fwrapv -O2 -Wall -g -fstack-protector-strong'
         ' -Wformat -Werror=format-security -Wdate-time -D_FORTIFY_SOURCE=2 -fPIC', '<G++>'),
        ('-I<QT-DIR>/include -I/usr/include/python3.6m', '<INC-STD>'),
    ]
indents = [
        (r"""^generating extension '.*' with variant '.*'$""", 0, True),
        (r"""building '.*' extension from C-Code$""", 0, True),
        (r"""^use """, 0, True),
        (r"""^\s+\d+x replaced '\\\\\[\\\\\[\w*\\\\\]\\\\\]' with .*$""", 4, False),
        (r'^generating .*.tmp$', 4, False),
        (r'^moving .*\.tmp -> .*$', 4, False),
        (r'^<GCC> ', 4, False),
        (r'^<G\+\+> ', 4, False),
        (r'^<G\+\+SO> ', 4, False),
    ]

def extract_files(line):
    def split_replist(l):
        assert type(l) is str, l
        import ast
        try:
            val = ast.literal_eval(l)
        except (SyntaxError, ValueError):
            yield l
        else:
            if type(val) is str: yield val
            elif type(val) is tuple: yield from val
            elif type(val) is list: yield from val
            else: yield l
    toks = line.split()
    toks = toks[1:]
    toks = [t.rstrip(',') for t in toks if t[0]!='-' and '/' in t]
    toks = [t for l in toks for t in split_replist(l)]
    return toks
    
def files_subset(cfiles, pfiles):
    if not cfiles:
        return False
    res = True
    hasfile = False
    for c in cfiles:
        if c in pfiles:
            hasfile = True
            continue
        for p in pfiles:
            if p.startswith(c+'/'):
                break
        else:
            return False
    return hasfile
    
def indent(lines):
    gindent = 0
    pfiles = []
    for line in lines:
        cfiles = extract_files(line)
        if files_subset(cfiles, pfiles):
            xindent = 4
        else:
            pfiles = cfiles
            xindent = 0
        for reg, lindent, header in indents:
            m = re.match(reg, line, flags=re.A)
            if m is not None:
                if header:
                    gindent = lindent + 4
                    pfiles = []
                    yield ' '*lindent + line
                else:
                    yield ' '*(gindent+lindent) + line
                break
        else:
            yield ' '*(gindent+xindent) + line
            
def replace_text(lines):
    yield 'Log filter replaced text:\n'
    maxtargetlen = max(len(t) for s,t in replacements) + 1
    for s, t in replacements:
        yield f'    {t+":":{maxtargetlen}} {s}\n'
    for line in lines:
        for s, t in replacements:
            line = line.replace(s, t)
        yield line
        
def main():
    lines = sys.stdin
    lines = indent(lines)
    lines = replace_text(lines)
    for line in lines:
        print(line, end='')
        
if __name__ == '__main__':
    main()
    

