#!/usr/bin/python3
# -*- coding: utf-8 -*-

#  Copyright © 2015  B. Clausius <barcc@gmx.de>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os
from xml.etree import ElementTree

def usage(exitcode, message=None):
    if message is not None:
        print(message)
        print()
    print('Usage:', sys.argv[0], '<filename with qmlprofiler data>')
    sys.exit(exitcode)
    
def process(xroot, flags):
    events = {}
    rows = []
    print(xroot.tag)
    for xchild in xroot:
        print(xchild.tag, xchild.attrib)
        if xchild.tag == 'eventData':
            for xevent in xchild:
                #print(xevent.tag, xevent.attrib)
                events[xevent.attrib['index']] = event = {}
                for xitem in xevent:
                    event[xitem.tag] = xitem.text
                #print(event)
        if xchild.tag == 'profilerDataModel':
            for xrow in xchild:
                #print(xrow.tag, xrow.attrib)
                rows.append(xrow.attrib)
    from warnings import warn_explicit
    class QmlProfiler (Warning): pass
    for row in rows:
        event = events[row['eventIndex']]
        eurl = event.get('filename')
        if eurl is None or not eurl.startswith('file:///'):
            continue
        efile = eurl[7:]
        cwd = os.getcwd()
        if 'cwd' in flags and not os.path.samefile(cwd, os.path.commonprefix([cwd, efile])):
            continue
        ecategory = event['displayname']
        edesc = event['details'] or '?'
        etype = event['type'] + event.get('bindingType','')
        ewarning = QmlProfiler('{}: ({}) {}'.format(ecategory, etype, edesc))
        eline = int(event.get('line', 0))
        emodule = os.path.basename(efile)
        print('{2}:{3}:{4}: {0}'.format(ewarning, edesc, efile, eline, event.get('column', '')), file=sys.stderr)
        warn_explicit(ewarning, None, efile, eline, module=emodule)
    
def main():
    args = sys.argv[1:]
    #if len(args) != 1: usage(1, "needs one argument")
    flags = set()
    filename = None
    for arg in args:
        if arg in ('-h', '--help'): usage(0)
        if arg == '--cwd':
            flags.add(arg[2:])
        elif filename is None:
            filename = arg
        else:
            usage(1, 'unexpected argument: {!r}'.format(arg))
    if filename is None:
        usage(1, 'missing argument')
    process(ElementTree.parse(filename).getroot(), flags)
    
    
if __name__ == '__main__':
    main()
    

